@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*               Copyright 2002 by Ullrich Koethe                       */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -1027,8 +1026,7 @@ namespace {
         pimpl->components = numBands;
     }
 
-    void GIFEncoder::setCompressionType( const std::string & comp,
-                                         int quality )
+    void GIFEncoder::setCompressionType( const std::string & /* comp */, int /* quality */)
     {
         VIGRA_IMPEX_FINALIZED(pimpl->finalized);
     }
