@@ -321,8 +321,8 @@ bool isImage(char const * filename)
 
 // class ImageExportInfo
 
-ImageExportInfo::ImageExportInfo( const char * filename )
-    : m_filename(filename),
+ImageExportInfo::ImageExportInfo( const char * filename, const char * mode )
+    : m_filename(filename), m_mode(mode),
       m_x_res(0), m_y_res(0),
       fromMin_(0.0), fromMax_(0.0), toMin_(0.0), toMax_(0.0)
 {}
@@ -389,6 +389,11 @@ const char * ImageExportInfo::getFileName() const
     return m_filename.c_str();
 }
 
+const char * ImageExportInfo::getMode() const
+{
+    return m_mode.c_str();
+}
+
 const char * ImageExportInfo::getFileType() const
 {
     return m_filetype.c_str();
@@ -438,6 +443,17 @@ ImageExportInfo & ImageExportInfo::setPosition(const vigra::Diff2D & pos)
     return *this;
 }
 
+vigra::Size2D ImageExportInfo::getCanvasSize() const
+{
+    return m_canvas_size ;
+}
+
+ImageExportInfo & ImageExportInfo::setCanvasSize(const Size2D & size)
+{
+    m_canvas_size = size;
+    return *this;
+}
+
 vigra::Diff2D ImageExportInfo::getPosition() const
 {
     return m_pos;
@@ -464,27 +480,41 @@ std::auto_ptr<Encoder> encoder( const ImageExportInfo & info )
     if ( filetype != "" ) {
         validate_filetype(filetype);
         std::auto_ptr<Encoder> enc2
-            = getEncoder( std::string( info.getFileName() ), filetype );
+            = getEncoder( std::string( info.getFileName() ), filetype, std::string( info.getMode() ) );
         enc = enc2;
     } else {
         std::auto_ptr<Encoder> enc2
-            = getEncoder( std::string( info.getFileName() ) );
+            = getEncoder( std::string( info.getFileName() ), "undefined", std::string( info.getMode() ) );
         enc = enc2;
     }
 
     std::string comp = info.getCompression();
     if ( comp != "" ) {
 
-        // check for JPEG compression
+        // check for quality parameter of JPEG compression
         int quality = -1;
-        std::istringstream compstream(comp.c_str());
-        compstream >> quality;
 
-        // FIXME: dangelo: This code might lead to strange effects (setting an invalid compression mode),
-        // if other formats also support a numerical compression parameter.
-        if ( quality != -1 ) {
-            enc->setCompressionType( "JPEG", quality );
-        } else {
+        // possibility 1: quality specified as "JPEG QUALITY=N" or "JPEG-ARITH QUALITY=N"
+        // possibility 2 (deprecated): quality specified as just a number "10"
+        std::string sq(" QUALITY="), parsed_comp;
+        std::string::size_type pos = comp.rfind(sq), start = 0;
+
+        if(pos != std::string::npos)
+        {
+            start = pos + sq.size();
+            parsed_comp = comp.substr(0, pos);
+        }
+
+        std::istringstream compstream(comp.substr(start));
+        compstream >> quality;
+        if ( quality != -1 )
+        {
+            if(parsed_comp == "")
+                parsed_comp = "JPEG";
+             enc->setCompressionType( parsed_comp, quality );
+        }
+        else
+        {
             // leave any other compression type to the codec
             enc->setCompressionType(comp);
         }
@@ -506,6 +536,7 @@ std::auto_ptr<Encoder> encoder( const ImageExportInfo & info )
     enc->setXResolution(info.getXResolution());
     enc->setYResolution(info.getYResolution());
     enc->setPosition(info.getPosition());
+    enc->setCanvasSize(info.getCanvasSize());
 
     if ( info.getICCProfile().size() > 0 ) {
         enc->setICCProfile(info.getICCProfile());
@@ -516,24 +547,10 @@ std::auto_ptr<Encoder> encoder( const ImageExportInfo & info )
 
 // class ImageImportInfo
 
-ImageImportInfo::ImageImportInfo( const char * filename )
-    : m_filename(filename)
+ImageImportInfo::ImageImportInfo( const char * filename, unsigned int imageIndex )
+    : m_filename(filename), m_image_index(imageIndex)
 {
-    std::auto_ptr<Decoder> decoder = getDecoder(m_filename);
-
-    m_filetype = decoder->getFileType();
-    m_pixeltype = decoder->getPixelType();
-    m_width = decoder->getWidth();
-    m_height = decoder->getHeight();
-    m_num_bands = decoder->getNumBands();
-    m_num_extra_bands = decoder->getNumExtraBands();
-    m_pos = decoder->getPosition();
-    m_x_res = decoder->getXResolution();
-    m_y_res = decoder->getYResolution();
-
-    m_icc_profile = decoder->getICCProfile();
-
-    decoder->abort(); // there probably is no better way than this
+    readHeader_();
 }
 
 ImageImportInfo::~ImageImportInfo() {
@@ -595,6 +612,22 @@ int ImageImportInfo::numExtraBands() const
     return m_num_extra_bands;
 }
 
+int ImageImportInfo::numImages() const
+{
+    return m_num_images;
+}
+
+void ImageImportInfo::setImageIndex(int index)
+{
+    m_image_index = index;
+    readHeader_();
+}
+
+int ImageImportInfo::getImageIndex() const
+{
+    return m_image_index;
+}
+
 Size2D ImageImportInfo::size() const
 {
     return Size2D( m_width, m_height );
@@ -625,6 +658,11 @@ Diff2D ImageImportInfo::getPosition() const
     return m_pos;
 }
 
+Size2D ImageImportInfo::getCanvasSize() const
+{
+    return m_canvas_size;
+}
+
 float ImageImportInfo::getXResolution() const
 {
     return m_x_res;
@@ -640,12 +678,33 @@ const ImageImportInfo::ICCProfile & ImageImportInfo::getICCProfile() const
     return m_icc_profile;
 }
 
+void ImageImportInfo::readHeader_()
+{
+    std::auto_ptr<Decoder> decoder = getDecoder(m_filename, "undefined", m_image_index);
+    m_num_images = decoder->getNumImages();
+
+    m_filetype = decoder->getFileType();
+    m_pixeltype = decoder->getPixelType();
+    m_width = decoder->getWidth();
+    m_height = decoder->getHeight();
+    m_num_bands = decoder->getNumBands();
+    m_num_extra_bands = decoder->getNumExtraBands();
+    m_pos = decoder->getPosition();
+    m_canvas_size = decoder->getCanvasSize();
+    m_x_res = decoder->getXResolution();
+    m_y_res = decoder->getYResolution();
+
+    m_icc_profile = decoder->getICCProfile();
+
+    decoder->abort(); // there probably is no better way than this
+}
+
 // return a decoder for a given ImageImportInfo object
 std::auto_ptr<Decoder> decoder( const ImageImportInfo & info )
 {
     std::string filetype = info.getFileType();
     validate_filetype(filetype);
-    return getDecoder( std::string( info.getFileName() ), filetype );
+    return getDecoder( std::string( info.getFileName() ), filetype, info.getImageIndex() );
 }
 
 // class VolumeExportInfo
@@ -668,7 +727,7 @@ VolumeExportInfo & VolumeExportInfo::setFileType( const char * filetype )
 }
 
 VolumeExportInfo & VolumeExportInfo::setForcedRangeMapping(double fromMin, double fromMax,
-                                                     double toMin, double toMax) 
+                                                     double toMin, double toMax)
 {
     fromMin_ = fromMin;
     fromMax_ = fromMax;
