@@ -226,8 +226,9 @@ struct ConvolutionTest
 
     }
 
-    void gaussianSharpeningTest(){
-         Image dest_lenna(lenna);
+    void gaussianSharpeningTest()
+    {
+        Image dest_lenna(lenna);
         gaussianSharpening(srcImageRange(lenna), destImage(dest_lenna), 3., 0.7);
 
         Image dest_correct;
@@ -241,7 +242,8 @@ struct ConvolutionTest
         Image::iterator i_dest_correct = dest_correct.begin();
         Image::Accessor acc_dest_correct = dest_correct.accessor();
 
-        for (; i_dest != end_dest; i_dest++, i_dest_correct++ ){
+        for (; i_dest != end_dest; i_dest++, i_dest_correct++ )
+        {
             shouldEqualTolerance(acc_dest(i_dest) , acc_dest_correct(i_dest_correct), 1e-6);
         }
 
@@ -302,7 +304,8 @@ struct ConvolutionTest
     }
     
 
-    void stdConvolutionTestWithAvoid(){
+    void stdConvolutionTestWithAvoid()
+    {
         Image dest(sym_image);
         dest.init(42.1);
 
@@ -320,22 +323,21 @@ struct ConvolutionTest
         for(int y = 0; i_dest != i_dest_end; y++){
             for(int x = 0; x < dest.size().x; x++, ++i_dest){
                 if(x == 0 || y == 0 || x == 9 || y == 9){
-                    should(acc(i_dest) == 42.1);
+                    shouldEqual(acc(i_dest), 42.1);
                 }
             }
         }
 
     }
     
-    void stdConvolutionTestWithClip(){
+    void stdConvolutionTestWithClip()
+    {
         Image dest(sym_image);
         dest.init(42.1);
 
         convolveImage(srcImageRange(sym_image), destImage(dest), kernel2d(unsym_kernel, BORDER_TREATMENT_CLIP));
 
         Image::Iterator i_dest_2D = dest.upperLeft();
-        Image::ScanOrderIterator i_dest = dest.begin();
-        Image::ScanOrderIterator i_dest_end = dest.end();
         Image::Accessor acc = dest.accessor();
 
         //Kontrollierung der Randbehandlung und ein paar Pixel
@@ -353,15 +355,14 @@ struct ConvolutionTest
         shouldEqualTolerance (acc(i_dest_2D + Diff2D(6,4)), 9.14363,   1E-4);
     }
     
-    void stdConvolutionTestWithWrap(){
+    void stdConvolutionTestWithWrap()
+    {
         Image dest(unsym_image);
         dest.init(42.1);
 
         convolveImage(srcImageRange(unsym_image), destImage(dest), kernel2d(unsym_kernel, BORDER_TREATMENT_WRAP));
 
         Image::Iterator i_dest_2D = dest.upperLeft();
-        Image::ScanOrderIterator i_dest = dest.begin();
-        Image::ScanOrderIterator i_dest_end = dest.end();
         Image::Accessor acc = dest.accessor();
 
         //Kontrollierung der Randbehandlung und ein paar Pixel
@@ -379,15 +380,14 @@ struct ConvolutionTest
         shouldEqualTolerance (acc(i_dest_2D + Diff2D(6,4)), 48.4841,   1E-5);
     }
     
-    void stdConvolutionTestWithReflect(){
+    void stdConvolutionTestWithReflect()
+    {
         Image dest(unsym_image);
         dest.init(42.1);
 
         convolveImage(srcImageRange(unsym_image), destImage(dest), kernel2d(unsym_kernel, BORDER_TREATMENT_REFLECT));
 
         Image::Iterator i_dest_2D = dest.upperLeft();
-        Image::ScanOrderIterator i_dest = dest.begin();
-        Image::ScanOrderIterator i_dest_end = dest.end();
         Image::Accessor acc = dest.accessor();
 
         //Kontrollierung der Randbehandlung und ein paar Pixel
@@ -405,15 +405,14 @@ struct ConvolutionTest
         shouldEqualTolerance (acc(i_dest_2D + Diff2D(6,4)), 48.4841,   1E-5);
     }
     
-    void stdConvolutionTestWithRepeat(){
+    void stdConvolutionTestWithRepeat()
+    {
         Image dest(unsym_image);
         dest.init(42.1);
 
         convolveImage(srcImageRange(unsym_image), destImage(dest), kernel2d(unsym_kernel, BORDER_TREATMENT_REPEAT));
 
         Image::Iterator i_dest_2D = dest.upperLeft();
-        Image::ScanOrderIterator i_dest = dest.begin();
-        Image::ScanOrderIterator i_dest_end = dest.end();
         Image::Accessor acc = dest.accessor();
 
         //Kontrollierung der Randbehandlung und ein paar Pixel
@@ -431,7 +430,8 @@ struct ConvolutionTest
         shouldEqualTolerance (acc(i_dest_2D + Diff2D(6,4)), 48.4841,   1E-5);
     }
 
-    void stdConvolutionTestFromWrapWithReflect(){
+    void stdConvolutionTestFromWrapWithReflect()
+    {
     
         Image src_wrap(78, 1);
         Image src_reflect(40, 1);
@@ -467,7 +467,8 @@ struct ConvolutionTest
         }
     }
   
-    void stdConvolutionTestFromRepeatWithAvoid(){
+    void stdConvolutionTestFromRepeatWithAvoid()
+    {
         Image src_avoid(40, 1);
         src_avoid.init(2.47);
         Image src_repeat(36, 1);
@@ -753,8 +754,9 @@ struct ConvolutionTest
     
     void gaussianSmoothingTest()
     {
+        double scale = 1.0;
         vigra::Kernel1D<double> gauss;
-        gauss.initGaussian(1.0);
+        gauss.initGaussian(scale);
         gauss.setBorderTreatment(BORDER_TREATMENT_REFLECT);
 
         Image tmp1(lenna.size());
@@ -763,7 +765,7 @@ struct ConvolutionTest
         separableConvolveX(srcImageRange(lenna), destImage(tmp1), kernel1d(gauss));
         separableConvolveY(srcImageRange(tmp1), destImage(tmp2), kernel1d(gauss));
         
-        gaussianSmoothing(srcImageRange(lenna), destImage(tmp1), 1.0);
+        gaussianSmoothing(srcImageRange(lenna), destImage(tmp1), scale);
         
         Image::ScanOrderIterator i1 = tmp1.begin();
         Image::ScanOrderIterator i1end = tmp1.end();
@@ -774,6 +776,24 @@ struct ConvolutionTest
         {
             should(acc(i1) == acc(i2));
         }
+
+        // test recursive approximation
+        Image recursive(lenna.size());
+        recursiveGaussianFilterX(srcImageRange(lenna), destImage(tmp2), scale);
+        recursiveGaussianFilterY(srcImageRange(tmp2), destImage(recursive), scale);
+
+        i1 = tmp1.begin();
+        i1end = tmp1.end();
+        i2 = recursive.begin();
+        
+        double sum = 0.0;
+        for(; i1 != i1end; ++i1, ++i2)
+        {
+            double diff = abs(acc(i1) - acc(i2));
+            sum += diff;
+            should(diff < 6.0);
+        }
+        should(sum / lenna.width() / lenna.height() < 0.5);
     }
     
     void optimalSmoothing3Test()
@@ -858,6 +878,19 @@ struct ConvolutionTest
 
             shouldEqualTolerance(grad-acc(i), 0.0, 1e-12);
         }
+
+        // compare with 2D convolution
+        vigra::Kernel2D<double> gradx;
+        gradx.initSeparable(grad, gauss);
+        Image nsgrad(lenna.size());
+        convolveImage(srcImageRange(lenna), destImage(nsgrad), kernel2d(gradx));
+
+        using namespace vigra::functor;
+
+        combineTwoImages(srcImageRange(nsgrad), srcImage(tmp1), destImage(nsgrad), Arg1() - Arg2());
+        Image zero(lenna.size(), 0.0);
+        shouldEqualSequenceTolerance(nsgrad.data(), nsgrad.data()+nsgrad.width()*nsgrad.height(), 
+                                     zero.data(), 1e-12);
     }
     
     void gradientTest()
@@ -1696,7 +1729,7 @@ struct ConvolutionTest
             }
             else
             {
-                should(acc_dest_avoid(dest_iter_avoid) == acc_dest_repeat(dest_iter_repeat));
+                shouldEqualTolerance(acc_dest_avoid(dest_iter_avoid), acc_dest_repeat(dest_iter_repeat), 1e-15);
                 dest_iter_repeat++;
             }
         }
