@@ -83,7 +83,11 @@ public:
         const std::string formats = impexListFormats();
         const std::string extensions = impexListExtensions();
 
-        const char * refFormats = "BMP GIF HDR "
+        const char * refFormats = "BMP "
+#if defined(HasEXR)
+        "EXR "
+#endif
+        "GIF HDR "
 #if defined(HasJPEG)
         "JPEG "
 #endif
@@ -97,7 +101,11 @@ public:
         "VIFF";
         shouldEqual(formats, refFormats);
 
-        const char * refExtensions = "bmp gif hdr "
+        const char * refExtensions = "bmp "
+#if defined(HasEXR)
+        "exr "
+#endif
+        "gif hdr "
 #if defined(HasJPEG)
         "jpeg jpg "
 #endif
@@ -154,7 +162,7 @@ public:
 #if !defined(HasJPEG)
         failCodec(img, exportinfo);
 #else
-        exportinfo.setCompression ("100");
+        exportinfo.setCompression ("JPEG QUALITY=100");
         exportImage (srcImageRange (img), exportinfo);
 
         vigra::ImageImportInfo info ("res.jpg");
@@ -209,6 +217,38 @@ public:
 #endif
     }
 
+    void testTIFFSequence()
+    {
+#if defined(HasTIFF)
+        for (int i=0; i < 3; ++i)
+        {
+            std::string fileName = std::string("lenna_") + vigra::asString(i) + ".tif";
+            vigra::ImageImportInfo ininfo (fileName.c_str());
+            Image inimg(ininfo.width(), ininfo.height());
+            importImage(ininfo, destImage(inimg));
+            vigra::ImageExportInfo outinfo ("resseq.tif", i==0?"w":"a");
+            exportImage(srcImageRange(inimg), outinfo);
+        }
+
+        for (int j=0; j < 3; ++j)
+        {
+            vigra::ImageImportInfo ininfo ("resseq.tif", j);
+            Image inimg(ininfo.width(), ininfo.height());
+            std::string fileName = std::string("lenna_") + vigra::asString(j) + ".tif";
+            importImage(ininfo, destImage(inimg));
+            vigra::ImageImportInfo originfo (fileName.c_str());
+            Image origimg(originfo.width(), originfo.height());
+            importImage(originfo, destImage(origimg));
+
+            Image::ScanOrderIterator it = inimg.begin ();
+            Image::ScanOrderIterator it1 = origimg.begin ();
+            Image::Accessor acc = inimg.accessor ();
+            for (; it != inimg.end (); ++it, ++it1)
+                should (acc (it) == acc (it1));
+        }
+#endif
+    }
+
     void testBMP ()
     {
         testFile ("res.bmp");
@@ -373,10 +413,10 @@ public:
     void testJPEG ()
     {
 #if !defined(HasJPEG)
-        failCodec(img, vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
+        failCodec(img, vigra::ImageExportInfo ("res.jpg").setCompression ("JPEG QUALITY=100"));
 #else
         exportImage (srcImageRange (img),
-                     vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
+                     vigra::ImageExportInfo ("res.jpg").setCompression ("JPEG QUALITY=100"));
 
         vigra::ImageImportInfo info ("res.jpg");
 
@@ -545,6 +585,29 @@ ByteRGBImageExportImportTest::testFile (const char *fileName)
         should (acc (i) == acc (i1));
 }
 
+class CanvasSizeTest
+{
+  public:
+    void testTIFFCanvasSize ()
+    {
+        vigra::ImageExportInfo exportinfo ("res.tif");
+        FRGBImage img(1, 1);
+#if !defined(HasTIFF)
+        failCodec(img, exportinfo);
+#else
+        img(0,0) = 1;
+        exportinfo.setCompression ("LZW");
+        Size2D canvasSize(3, 8);
+        exportinfo.setCanvasSize (canvasSize);
+        exportImage (srcImageRange (img), exportinfo);
+
+        vigra::ImageImportInfo info ("res.tif");
+
+        should (info.getCanvasSize () == canvasSize);
+#endif
+    }
+};
+
 class PNGInt16Test
 {
   public:
@@ -638,10 +701,10 @@ public:
     void testJPEG ()
     {
 #if !defined(HasJPEG)
-        failCodec(img, vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
+        failCodec(img, vigra::ImageExportInfo ("res.jpg").setCompression ("JPEG QUALITY=100"));
 #else
         exportImage (srcImageRange (img),
-                     vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
+                     vigra::ImageExportInfo ("res.jpg").setCompression ("JPEG QUALITY=100"));
 
         vigra::ImageImportInfo info ("res.jpg");
 
@@ -665,6 +728,36 @@ public:
 #endif
     }
 
+    void testPNG ()
+    {
+#if !defined(HasPNG)
+        failCodec(img, vigra::ImageExportInfo ("res.png"));
+#else
+        exportImage (srcImageRange (img),
+                     vigra::ImageExportInfo ("res_.png"));
+
+        vigra::ImageImportInfo info ("res_.png");
+
+        should (info.width () == reread.width ());
+        should (info.height () == reread.height ());
+        should (info.isGrayscale ());
+        should (info.getPixelType () == std::string ("UINT8"));
+
+        Image res (info.width (), info.height ());
+
+        importImage (info, destImage (res));
+
+        Image::ScanOrderIterator i = reread.begin ();
+        Image::ScanOrderIterator i1 = res.begin ();
+        Image::Accessor acc = reread.accessor ();
+
+        double sum = 0.0;
+        for (; i != reread.end (); ++i, ++i1)
+            sum += std::abs (acc (i) - acc (i1));
+        should (sum / (info.width () * info.height ()) < 0.1);
+#endif
+    }
+
     void testTIFF ()
     {
 #if !defined(HasTIFF)
@@ -824,10 +917,10 @@ public:
     void testJPEG ()
     {
 #if !defined(HasJPEG)
-        failCodec(img, vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
+        failCodec(img, vigra::ImageExportInfo ("res.jpg").setCompression ("JPEG QUALITY=100"));
 #else
         exportImage (srcImageRange (img),
-                     vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
+                     vigra::ImageExportInfo ("res.jpg").setCompression ("JPEG QUALITY=100"));
 
         vigra::ImageImportInfo info ("res.jpg");
 
@@ -1047,7 +1140,7 @@ public:
         }
     }
 
-    void failingTest (char const * filename, 
+    void failingTest (char const * filename,
                       char const * message = "exportImage(): file format does not support requested number of bands (color channels)")
     {
         try
@@ -1141,6 +1234,34 @@ public:
 #endif
     }
 
+    void testEXR ()
+    {
+#if !defined(HasEXR)
+        failCodec(img, vigra::ImageExportInfo ("res.exr"));
+#else
+        exportImage (srcImageRange (img),
+                     vigra::ImageExportInfo ("res.exr"));
+
+        vigra::ImageImportInfo info ("res.exr");
+
+        should (info.width () == img.width ());
+        should (info.height () == img.height ());
+        should (info.isColor ());
+        should (info.getPixelType () == std::string ("FLOAT"));
+
+        Image res (info.width (), info.height ());
+
+        importImage (info, destImage (res));
+
+        Image::ScanOrderIterator i = img.begin ();
+        Image::ScanOrderIterator i1 = res.begin ();
+        Image::Accessor acc = img.accessor ();
+
+        for (; i != img.end (); ++i, ++i1)
+            shouldEqual (acc (i), acc (i1));
+#endif
+    }
+
     void testPNG ()
     {
 #if !defined(HasPNG)
@@ -1225,6 +1346,17 @@ public:
         testImport("tiff");
     }
 
+    // exr
+
+    void testEXRExport()
+    {
+#if !defined(HasEXR)
+        testExport("exr", "did not find a matching codec for the given file extension");
+#else
+        testExport("exr");
+#endif
+    }
+
     // viff
 
     void testVIFFExport()
@@ -1349,6 +1481,7 @@ struct ImageImportExportTestSuite : public vigra::test_suite
         add(testCase(&ByteImageExportImportTest::testGIF));
         add(testCase(&ByteImageExportImportTest::testJPEG));
         add(testCase(&ByteImageExportImportTest::testTIFF));
+        add(testCase(&ByteImageExportImportTest::testTIFFSequence));
         add(testCase(&ByteImageExportImportTest::testBMP));
         add(testCase(&ByteImageExportImportTest::testPGM));
         add(testCase(&ByteImageExportImportTest::testPNM));
@@ -1376,9 +1509,12 @@ struct ImageImportExportTestSuite : public vigra::test_suite
         add(testCase(&PNGInt16Test::testByteOrder));
 #endif
 
+        add(testCase(&CanvasSizeTest::testTIFFCanvasSize));
+
         // grayscale float images
         add(testCase(&FloatImageExportImportTest::testGIF));
         add(testCase(&FloatImageExportImportTest::testJPEG));
+        add(testCase(&FloatImageExportImportTest::testPNG));
         add(testCase(&FloatImageExportImportTest::testTIFF));
         add(testCase(&FloatImageExportImportTest::testTIFFForcedRange));
         add(testCase(&FloatImageExportImportTest::testBMP));
@@ -1393,6 +1529,7 @@ struct ImageImportExportTestSuite : public vigra::test_suite
         add(testCase(&Vector4ExportImportTest::testSUN));
         add(testCase(&Vector4ExportImportTest::testVIFF));
         add(testCase(&Vector4ExportImportTest::testTIFF));
+        add(testCase(&Vector4ExportImportTest::testEXR));
         add(testCase(&Vector4ExportImportTest::testPNG));
 
         // rgb float images
@@ -1404,7 +1541,7 @@ struct ImageImportExportTestSuite : public vigra::test_suite
         add(testCase(&FloatRGBImageExportImportTest::testVIFF));
         add(testCase(&FloatRGBImageExportImportTest::testHDR));
 
-		// failure tests
+        // failure tests
         add(testCase(&ImageExportImportFailureTest::testGIFExport));
         add(testCase(&ImageExportImportFailureTest::testGIFImport));
         add(testCase(&ImageExportImportFailureTest::testJPEGExport));
@@ -1431,5 +1568,5 @@ int main (int argc, char ** argv)
     const int failed = test.run(vigra::testsToBeExecuted(argc, argv));
     std::cout << test.report() << std::endl;
 
-	return failed != 0;
+    return failed != 0;
 }
