@@ -1,6 +1,6 @@
 /************************************************************************/
 /*                                                                      */
-/*                 Copyright 2004 by Ullrich Koethe                     */
+/*              Copyright 2004-2010 by Ullrich Koethe                   */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
@@ -596,6 +596,20 @@ struct EdgeDetectionTest
             should(VIGRA_CSTD::fabs(edgels[i].orientation-M_PI*0.25) < 0.1);
         }
         should(count == 75);
+
+        std::vector<vigra::Edgel> edgelsThresh;
+        double threshold = 1.25;
+        cannyEdgelListThreshold(srcImageRange(imgCanny), edgelsThresh, 1.0, threshold);
+        count = 0;
+        for(unsigned int i=0; i<edgels.size(); ++i)
+        {
+            if (edgels[i].strength <= threshold)
+                continue;  // ignore edgels below threshold
+            should(edgels[i].x == edgelsThresh[count].x);
+            should(edgels[i].y == edgelsThresh[count].y);
+            ++count;
+        }
+        should(count == 38);
     }
 
     void cannyEdgelList3x3Test()
@@ -612,6 +626,20 @@ struct EdgeDetectionTest
             should(VIGRA_CSTD::fabs(edgels[i].orientation-M_PI*0.25) < 0.1);
         }
         should(count == 38);
+
+        std::vector<vigra::Edgel> edgelsThresh;
+        double threshold = 1.3;
+        cannyEdgelList3x3Threshold(srcImageRange(imgCanny), edgelsThresh, 1.0, threshold);
+        count = 0;
+        for(unsigned int i=0; i<edgels.size(); ++i)
+        {
+            if (edgels[i].strength <= threshold)
+                continue;  // ignore edgels below threshold
+            should(edgels[i].x == edgelsThresh[count].x);
+            should(edgels[i].y == edgelsThresh[count].y);
+            ++count;
+        }
+        should(count == 36);
     }
 
     void cannyEdgeImageTest()
@@ -782,20 +810,22 @@ struct EqualWithToleranceFunctor
 struct LocalMinMaxTest
 {
     typedef vigra::DImage Image;
+    typedef vigra::MultiArray<3,double> Volume;
+    typedef MultiArrayShape<3>::type Shp3D;
 
     LocalMinMaxTest()
-    : img(9,9)
+    : img(9,9), vol()
     {
         static const double in[] = {
-            0.0,  0.1,  0.1,  0.3,  0.5,  0.3,  0.0,  0.0, 0.0,
-            0.0, -0.1,  0.1,  0.0,  1.0,  0.0,  0.3,  0.0, 0.0,
-            0.0,  0.5,  2.0,  0.0,  2.0,  2.0,  2.0,  0.0, 0.0,
-            0.0,  0.0,  1.0,  1.5,  1.0,  1.0,  0.0,  0.0, 0.0,
-            0.0,  0.1,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0, 0.0,
-            0.0,  0.0,  0.0,  0.0, -1.0, -1.5, -1.0,  0.0, 0.0,
-            0.0,  0.0, -2.0, -2.0, -2.0,  0.0, -2.0, -0.5, 0.0,
-            0.0,  0.0,  0.0,  0.0, -1.0,  0.0, -0.1,  0.1, 0.0,
-            0.0,  0.0,  0.0,  0.0, -0.5, -0.3, -0.1, -0.1, 0.0};
+            0.2,  0.1,  0.1,  0.3,  0.5,  0.3,  0.0,  0.0, -0.1,
+            0.0, -0.1,  0.1,  0.0,  1.0,  0.0,  0.3,  0.0,  0.0,
+            0.0,  0.5,  2.0,  0.0,  2.0,  2.0,  2.0,  0.0, -1.1,
+            0.1,  0.0,  1.0,  1.5,  1.0,  1.0,  0.0,  0.0,  0.0,
+            0.0,  0.1,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,
+            0.0,  0.0,  0.0,  0.0, -1.0, -1.5, -1.0,  0.0, -0.3,
+            0.0,  0.0, -2.0, -2.0, -2.0,  0.0, -2.0, -0.5,  0.0,
+            1.0,  0.0,  0.0,  0.0, -1.0,  0.0, -0.1,  0.1,  0.0,
+            0.0,  0.0,  0.0,  0.0, -0.5, -0.3, -0.1, -0.1,  0.0};
 
         Image::ScanOrderIterator i = img.begin();
         Image::ScanOrderIterator end = img.end();
@@ -806,6 +836,170 @@ struct LocalMinMaxTest
         {
             acc.set(*p, i);
         }
+
+        //prepare the multiarray
+        vol.reshape(Shp3D(10,20,50),0);
+
+        vol(1,1,1)=10;
+        vol(5,5,5)=350;
+
+        vol(8,3,5)=9; //plateaux
+        vol(8,4,5)=9;
+        vol(8,5,5)=9;
+
+        vol(6,7,7)=-0.5;
+        vol(7,7,7)=-1;
+        vol(7,1,15)=-100;
+        vol(7,1,19)=-20;
+
+        vol(3,15,26)=-1; //plateaux
+        vol(3,15,27)=-1;
+        vol(3,15,28)=-1;
+        vol(3,16,26)=-1;
+
+        vol(9,18,35)=-100; //on the border is skipped
+        vol(0,1,49)=100; //on the border is skipped
+
+    }
+
+
+    void localMinimum3DTest()
+    {
+        Volume res(vol);
+        res.init(0);
+
+        localMinima3D(srcMultiArrayRange(vol), destMultiArray(res),1,NeighborCode3DSix());
+
+        Volume desired(vol);
+        desired.init(0);
+
+        desired(7,7,7)=1;
+        desired(7,1,15)=1;
+        desired(7,1,19)=1;
+
+
+        for(int z=0; z<vol.shape(2); ++z)
+            for(int y=0; y<vol.shape(1); ++y)
+                for(int x=0; x<vol.shape(0); ++x)
+                    shouldEqual(res(x,y,z), desired(x,y,z));
+
+    }
+
+    void extendedLocalMinimum3DTest()
+    {
+        Volume res(vol);
+        res.init(0);
+
+        extendedLocalMinima3D(srcMultiArrayRange(vol), destMultiArray(res),1,NeighborCode3DSix());
+
+        Volume desired(vol);
+        desired.init(0);
+
+        desired(7,7,7)=1;
+        desired(7,1,15)=1;
+        desired(7,1,19)=1;
+
+        desired(3,15,26)=1; //plateaux
+        desired(3,15,27)=1;
+        desired(3,15,28)=1;
+        desired(3,16,26)=1;
+
+        for(int z=0; z<vol.shape(2); ++z)
+            for(int y=0; y<vol.shape(1); ++y)
+                for(int x=0; x<vol.shape(0); ++x)
+                    shouldEqual(res(x,y,z), desired(x,y,z));
+
+    }
+    
+        void extendedLocalMinimum3DTest2()
+    {
+        Volume res(vol);
+        res.init(0);
+
+        extendedLocalMinima3D(srcMultiArrayRange(vol), destMultiArray(res),1,NeighborCode3DTwentySix());
+
+        Volume desired(vol);
+        desired.init(0);
+
+        desired(7,7,7)=1;
+        desired(7,1,15)=1;
+        desired(7,1,19)=1;
+
+        desired(3,15,26)=1; //plateaux
+        desired(3,15,27)=1;
+        desired(3,15,28)=1;
+        desired(3,16,26)=1;
+
+        for(int z=0; z<vol.shape(2); ++z)
+            for(int y=0; y<vol.shape(1); ++y)
+                for(int x=0; x<vol.shape(0); ++x)
+                    shouldEqual(res(x,y,z), desired(x,y,z));
+
+    }
+
+    void localMaximum3DTest()
+    {
+        Volume res(vol);
+        res.init(0);
+
+        localMaxima3D(srcMultiArrayRange(vol), destMultiArray(res),1,NeighborCode3DSix());
+
+        Volume desired(vol);
+        desired.init(0);
+
+        desired(1,1,1)=1;
+        desired(5,5,5)=1;
+
+        for(int z=0; z<vol.shape(2); ++z)
+            for(int y=0; y<vol.shape(1); ++y)
+                for(int x=0; x<vol.shape(0); ++x)
+                    shouldEqual(res(x,y,z), desired(x,y,z));
+    }
+
+    void extendedLocalMaximum3DTest()
+    {
+        Volume res(vol);
+        res.init(0);
+
+        extendedLocalMaxima3D(srcMultiArrayRange(vol), destMultiArray(res),1,NeighborCode3DSix());
+
+        Volume desired(vol);
+        desired.init(0);
+
+        desired(1,1,1)=1;
+        desired(5,5,5)=1;
+
+        desired(8,3,5)=1;
+        desired(8,4,5)=1;
+        desired(8,5,5)=1;
+
+        for(int z=0; z<vol.shape(2); ++z)
+            for(int y=0; y<vol.shape(1); ++y)
+                for(int x=0; x<vol.shape(0); ++x)
+                    shouldEqual(res(x,y,z), desired(x,y,z));
+    }
+    
+        void extendedLocalMaximum3DTest2()
+    {
+        Volume res(vol);
+        res.init(0);
+
+        extendedLocalMaxima3D(srcMultiArrayRange(vol), destMultiArray(res),1,NeighborCode3DTwentySix());
+
+        Volume desired(vol);
+        desired.init(0);
+
+        desired(1,1,1)=1;
+        desired(5,5,5)=1;
+
+        desired(8,3,5)=1;
+        desired(8,4,5)=1;
+        desired(8,5,5)=1;
+
+        for(int z=0; z<vol.shape(2); ++z)
+            for(int y=0; y<vol.shape(1); ++y)
+                for(int x=0; x<vol.shape(0); ++x)
+                    shouldEqual(res(x,y,z), desired(x,y,z));
     }
 
     void localMinimumTest()
@@ -813,9 +1007,9 @@ struct LocalMinMaxTest
         Image res(img);
         res.init(0);
 
-        localMinima(srcImageRange(img), destImage(res), 1.0);
+        localMinima(srcImageRange(img), destImage(res));
 
-        static const double desired[] = {
+        double desired[] = {
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
@@ -827,6 +1021,13 @@ struct LocalMinMaxTest
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
 
         shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMinima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().allowAtBorder());
+        desired[8] = 1.0;
+        desired[26] = 1.0;
+        shouldEqualSequence(res.begin(), res.end(), desired);
     }
 
     void localMinimum4Test()
@@ -834,13 +1035,13 @@ struct LocalMinMaxTest
         Image res(img);
         res.init(0);
 
-        localMinima(srcImageRange(img), destImage(res), 1.0, FourNeighborCode());
+        localMinima(srcImageRange(img), destImage(res), LocalMinmaxOptions().neighborhood(4));
 
-        static const double desired[] = {
+        double desired[] = {
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
-            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0,
@@ -848,6 +1049,42 @@ struct LocalMinMaxTest
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
 
         shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMinima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().neighborhood(4).allowAtBorder());
+        desired[8] = 1.0;
+        desired[26] = 1.0;
+        desired[53] = 1.0;
+        shouldEqualSequence(res.begin(), res.end(), desired);
+    }
+
+    void localMinimumTestThr()
+    {
+        Image res(img);
+        res.init(0);
+
+        localMinima(srcImageRange(img), destImage(res),
+                    LocalMinmaxOptions().neighborhood(8).markWith(1.0).threshold(-1.0));
+
+        double desired[] = {
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
+
+        shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMinima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().neighborhood(8).threshold(-1.0).allowAtBorder());
+        desired[26] = 1.0;
+        shouldEqualSequence(res.begin(), res.end(), desired);
     }
 
     void localMaximumTest()
@@ -855,9 +1092,9 @@ struct LocalMinMaxTest
         Image res(img);
         res.init(0);
 
-        localMaxima(srcImageRange(img), destImage(res), 1.0);
+        localMaxima(srcImageRange(img), destImage(res));
 
-        static const double desired[] = {
+        double desired[] = {
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
@@ -869,6 +1106,13 @@ struct LocalMinMaxTest
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
 
         shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMaxima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().allowAtBorder());
+        desired[0] = 1.0;
+        desired[63] = 1.0;
+        shouldEqualSequence(res.begin(), res.end(), desired);
     }
 
     void localMaximum4Test()
@@ -876,9 +1120,10 @@ struct LocalMinMaxTest
         Image res(img);
         res.init(0);
 
-        localMaxima(srcImageRange(img), destImage(res), 1.0, FourNeighborCode());
+        localMaxima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().neighborhood(4));
 
-        static const double desired[] = {
+        double desired[] = {
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
@@ -890,6 +1135,42 @@ struct LocalMinMaxTest
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
 
         shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMaxima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().neighborhood(4).allowAtBorder());
+        desired[0] = 1.0;
+        desired[27] = 1.0;
+        desired[63] = 1.0;
+        shouldEqualSequence(res.begin(), res.end(), desired);
+    }
+
+    void localMaximumTestThr()
+    {
+        Image res(img);
+        res.init(0);
+
+        localMaxima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().markWith(1.0).neighborhood(8).threshold(0.2));
+
+        double desired[] = {
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
+
+        shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMaxima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().allowAtBorder().threshold(0.2));
+        desired[63] = 1.0;
+        shouldEqualSequence(res.begin(), res.end(), desired);
     }
 
     void extendedLocalMinimumTest()
@@ -899,7 +1180,7 @@ struct LocalMinMaxTest
 
         extendedLocalMinima(srcImageRange(img), destImage(res), 1.0);
 
-        static const double desired[] = {
+        double desired[] = {
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 1.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0,
@@ -911,6 +1192,18 @@ struct LocalMinMaxTest
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
 
         shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMinima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().allowPlateaus());
+        shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMinima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().allowAtBorder().allowPlateaus());
+        desired[8] = 1.0;
+        desired[26] = 1.0;
+        shouldEqualSequence(res.begin(), res.end(), desired);
     }
 
     void extendedLocalMinimum4Test()
@@ -920,11 +1213,11 @@ struct LocalMinMaxTest
 
         extendedLocalMinima(srcImageRange(img), destImage(res), 1.0, FourNeighborCode());
 
-        static const double desired[] = {
+        double desired[] = {
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0,
-            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 1.0, 1.0, 1.0, 0.0, 1.0, 0.0, 0.0,
@@ -932,7 +1225,20 @@ struct LocalMinMaxTest
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
 
         shouldEqualSequence(res.begin(), res.end(), desired);
-    }
+ 
+        res.init(0);
+        localMinima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().allowPlateaus().neighborhood(4));
+        shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMinima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().neighborhood(4).allowAtBorder().allowPlateaus());
+        desired[8] = 1.0;
+        desired[26] = 1.0;
+        desired[53] = 1.0;
+        shouldEqualSequence(res.begin(), res.end(), desired);
+   }
 
     void extendedLocalMaximumTest()
     {
@@ -941,7 +1247,7 @@ struct LocalMinMaxTest
 
         extendedLocalMaxima(srcImageRange(img), destImage(res), 1.0);
 
-        static const double desired[] = {
+        double desired[] = {
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 1.0, 0.0, 1.0, 1.0, 1.0, 0.0, 0.0,
@@ -953,6 +1259,17 @@ struct LocalMinMaxTest
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
 
         shouldEqualSequence(res.begin(), res.end(), desired);
+        res.init(0);
+        localMaxima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().allowPlateaus());
+        shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMaxima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().allowAtBorder().allowPlateaus());
+        desired[0] = 1.0;
+        desired[63] = 1.0;
+        shouldEqualSequence(res.begin(), res.end(), desired);
    }
 
     void extendedLocalMaximum4Test()
@@ -962,7 +1279,7 @@ struct LocalMinMaxTest
 
         extendedLocalMaxima(srcImageRange(img), destImage(res), 1.0, FourNeighborCode());
 
-        static const double desired[] = {
+        double desired[] = {
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 1.0, 0.0, 1.0, 1.0, 1.0, 0.0, 0.0,
@@ -974,6 +1291,19 @@ struct LocalMinMaxTest
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
 
         shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMaxima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().allowPlateaus().neighborhood(4));
+        shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        localMaxima(srcImageRange(img), destImage(res), 
+                    LocalMinmaxOptions().neighborhood(4).allowAtBorder().allowPlateaus());
+        desired[0] = 1.0;
+        desired[27] = 1.0;
+        desired[63] = 1.0;
+        shouldEqualSequence(res.begin(), res.end(), desired);
    }
 
     void plateauWithHolesTest()
@@ -1011,6 +1341,7 @@ struct LocalMinMaxTest
     }
 
     Image img;
+    Volume vol;
 };
 
 struct WatershedsTest
@@ -1038,14 +1369,14 @@ struct WatershedsTest
 
         for(; i != end; ++i, ++p)
         {
-            acc.set(*p, i);
+            // transform data to a range suitable for BucketQueue (in the turbo algorithm)
+            acc.set(*p*10.0 + 30.0, i);
         }
     }
 
     void watershedsTest()
     {
-        Image res(img);
-        res.init(0);
+        IImage res(img.size());
 
         static const double desired[] = {
             1.0,  1.0,  1.0,  2.0,  3.0,  3.0,  3.0,  3.0,  4.0,
@@ -1058,28 +1389,98 @@ struct WatershedsTest
             7.0,  5.0,  5.0,  5.0,  5.0,  6.0,  6.0,  6.0,  6.0,
             7.0,  7.0,  7.0,  5.0,  5.0,  5.0,  5.0,  5.0,  5.0};
 
-        int count = watersheds(srcImageRange(img), destImage(res));
+        int count = watershedsUnionFind(srcImageRange(img), destImage(res));
+        
+        shouldEqual(7, count);
+        shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        count = generateWatershedSeeds(srcImageRange(img), destImage(res),
+                                       SeedOptions().extendedMinima());
+
+        static const double desiredSeeds[] = {
+            0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  1.0,  1.0,  1.0,  
+            0.0,  2.0,  0.0,  3.0,  0.0,  1.0,  0.0,  1.0,  1.0,  
+            0.0,  0.0,  0.0,  3.0,  0.0,  0.0,  0.0,  1.0,  1.0,  
+            0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  1.0,  
+            0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  
+            0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  
+            0.0,  0.0,  4.0,  4.0,  4.0,  0.0,  5.0,  0.0,  0.0,  
+            0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  
+            0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0};
+
+        shouldEqual(5, count);
+        shouldEqualSequence(res.begin(), res.end(), desiredSeeds);
+
+        count = watershedsRegionGrowing(srcImageRange(img), destImage(res));
+
+        static const double desiredRG[] = {
+            2.0,  2.0,  2.0,  3.0,  3.0,  1.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  3.0,  3.0,  1.0,  1.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  3.0,  3.0,  3.0,  1.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  3.0,  3.0,  3.0,  3.0,  1.0,  1.0,  1.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  5.0,  1.0,  1.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  5.0,  5.0,  5.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  5.0,  5.0,  5.0,  5.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  5.0,  5.0,  5.0,  5.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  5.0,  5.0,  5.0};
+
+        shouldEqual(5, count);
+        shouldEqualSequence(res.begin(), res.end(), desiredRG);
+
+        res.init(0);
+        count = watershedsRegionGrowing(srcImageRange(img), destImage(res),
+                                        WatershedOptions().keepContours()
+                                          .seedOptions(SeedOptions().extendedMinima()));
+
+        static const double desiredRGC[] = {
+            2.0,  2.0,  0.0,  3.0,  0.0,  1.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  0.0,  3.0,  0.0,  1.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  0.0,  3.0,  0.0,  1.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  1.0,  
+            0.0,  0.0,  0.0,  4.0,  4.0,  0.0,  5.0,  0.0,  0.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  0.0,  5.0,  5.0,  5.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  0.0,  5.0,  5.0,  5.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  0.0,  0.0,  0.0,  0.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0};
+
+        shouldEqual(5, count);
+        shouldEqualSequence(res.begin(), res.end(), desiredRGC);
+
+        res.init(0);
+        count = watershedsRegionGrowing(srcImageRange(img), destImage(res),
+                                        WatershedOptions().turboAlgorithm()
+                                          .seedOptions(SeedOptions().extendedMinima()));
+
+        static const double desiredTRG[] = {
+            2.0,  2.0,  2.0,  3.0,  3.0,  1.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  2.0,  3.0,  3.0,  1.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  2.0,  3.0,  3.0,  1.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  3.0,  3.0,  3.0,  5.0,  1.0,  1.0,  1.0,  
+            4.0,  4.0,  4.0,  4.0,  5.0,  5.0,  5.0,  5.0,  1.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  5.0,  5.0,  5.0,  5.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  5.0,  5.0,  5.0,  5.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  5.0,  5.0,  5.0,  5.0,  
+            4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  5.0,  5.0};
+
+        shouldEqual(5, count);
+        shouldEqualSequence(res.begin(), res.end(), desiredTRG);
 
 #if 0
         std::cerr << count << "\n";
         for(int y=0;y<9;++y)
         {
+            std::cerr << "            ";
             for(int x=0;x<9;++x)
-                std::cerr << res(x,y) << "     ";
-            std::cerr << "\n";
+                std::cerr << res(x,y) << ".0,  ";
+            std::cerr << "\n\n";
         }
 #endif /* #if 0 */
-
-        
-        should(7 == count);
-
-        shouldEqualSequence(res.begin(), res.end(), desired);
     }
 
     void watersheds4Test()
     {
-        Image res(img);
-        res.init(0);
+        IImage res(img.size());
 
         static const double desired[] = {
             1.0,  1.0,  1.0,  2.0,  2.0,  3.0,  4.0,  4.0,  5.0,
@@ -1092,21 +1493,93 @@ struct WatershedsTest
            10.0, 10.0,  7.0,  7.0,  7.0,  7.0,  9.0,  9.0,  7.0,
            10.0, 10.0, 10.0,  7.0,  7.0,  7.0,  7.0,  7.0,  7.0};
 
-        int count = watersheds(srcImageRange(img), destImage(res), FourNeighborCode());
+        int count = watershedsUnionFind(srcImageRange(img), destImage(res), FourNeighborCode());
+
+        should(10 == count);
+        shouldEqualSequence(res.begin(), res.end(), desired);
+
+        res.init(0);
+        count = generateWatershedSeeds(srcImageRange(img), destImage(res), FourNeighborCode(),
+                                       SeedOptions().extendedMinima());
+
+        static const double desiredSeeds[] = {
+            0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  1.0,  1.0,  1.0,  
+            0.0,  2.0,  0.0,  3.0,  0.0,  4.0,  0.0,  1.0,  1.0,  
+            0.0,  0.0,  0.0,  3.0,  0.0,  0.0,  0.0,  1.0,  1.0,  
+            0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  1.0,  
+            0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  
+            0.0,  0.0,  0.0,  0.0,  0.0,  5.0,  0.0,  0.0,  0.0,  
+            0.0,  0.0,  6.0,  6.0,  6.0,  0.0,  7.0,  0.0,  0.0,  
+            0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  
+            0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0};
+
+        shouldEqual(7, count);
+        shouldEqualSequence(res.begin(), res.end(), desiredSeeds);
+
+        count = watershedsRegionGrowing(srcImageRange(img), destImage(res), FourNeighborCode());
+
+        static const double desiredRG[] = {
+            2.0,  2.0,  2.0,  3.0,  3.0,  1.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  3.0,  3.0,  4.0,  4.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  3.0,  3.0,  3.0,  4.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  2.0,  3.0,  5.0,  5.0,  1.0,  1.0,  1.0,  
+            6.0,  6.0,  6.0,  6.0,  5.0,  5.0,  5.0,  1.0,  1.0,  
+            6.0,  6.0,  6.0,  6.0,  5.0,  5.0,  5.0,  7.0,  7.0,  
+            6.0,  6.0,  6.0,  6.0,  6.0,  7.0,  7.0,  7.0,  7.0,  
+            6.0,  6.0,  6.0,  6.0,  6.0,  6.0,  7.0,  7.0,  7.0,  
+            6.0,  6.0,  6.0,  6.0,  6.0,  6.0,  7.0,  7.0,  7.0};
+
+        shouldEqual(7, count);
+        shouldEqualSequence(res.begin(), res.end(), desiredRG);
+
+        res.init(0);
+        count = watershedsRegionGrowing(srcImageRange(img), destImage(res), FourNeighborCode(),
+                                        WatershedOptions().keepContours()
+                                          .seedOptions(SeedOptions().extendedMinima()));
+
+        static const double desiredRGC[] = {
+            2.0,  2.0,  2.0,  0.0,  0.0,  0.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  0.0,  3.0,  0.0,  4.0,  0.0,  1.0,  1.0,  
+            2.0,  2.0,  0.0,  3.0,  0.0,  0.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  2.0,  0.0,  5.0,  5.0,  0.0,  1.0,  1.0,  
+            0.0,  0.0,  0.0,  0.0,  5.0,  5.0,  5.0,  0.0,  0.0,  
+            6.0,  6.0,  6.0,  6.0,  0.0,  5.0,  0.0,  7.0,  7.0,  
+            6.0,  6.0,  6.0,  6.0,  6.0,  0.0,  7.0,  7.0,  7.0,  
+            6.0,  6.0,  6.0,  6.0,  6.0,  0.0,  7.0,  7.0,  7.0,  
+            6.0,  6.0,  6.0,  6.0,  6.0,  6.0,  0.0,  7.0,  7.0};
+
+        shouldEqual(7, count);
+        shouldEqualSequence(res.begin(), res.end(), desiredRGC);
+
+        res.init(0);
+        count = watershedsRegionGrowing(srcImageRange(img), destImage(res), FourNeighborCode(),
+                                        WatershedOptions().turboAlgorithm()
+                                          .seedOptions(SeedOptions().extendedMinima()));
+
+        static const double desiredTRG[] = {
+            2.0,  2.0,  2.0,  3.0,  1.0,  1.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  2.0,  3.0,  3.0,  4.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  3.0,  3.0,  3.0,  4.0,  1.0,  1.0,  1.0,  
+            2.0,  2.0,  2.0,  3.0,  6.0,  5.0,  7.0,  1.0,  1.0,  
+            6.0,  6.0,  6.0,  6.0,  6.0,  5.0,  7.0,  7.0,  1.0,  
+            6.0,  6.0,  6.0,  6.0,  6.0,  5.0,  7.0,  7.0,  7.0,  
+            6.0,  6.0,  6.0,  6.0,  6.0,  7.0,  7.0,  7.0,  7.0,  
+            6.0,  6.0,  6.0,  6.0,  6.0,  6.0,  7.0,  7.0,  7.0,  
+            6.0,  6.0,  6.0,  6.0,  6.0,  6.0,  6.0,  6.0,  6.0};
+
+        shouldEqual(7, count);
+        shouldEqualSequence(res.begin(), res.end(), desiredTRG);
 
 #if 0
         std::cerr << count << "\n";
         for(int y=0;y<9;++y)
         {
+            std::cerr << "            ";
             for(int x=0;x<9;++x)
-                std::cerr << res(x,y) << "     ";
+                std::cerr << res(x,y) << ".0,  ";
             std::cerr << "\n";
         }
 #endif /* #if 0 */
-
-        should(10 == count);
-
-        shouldEqualSequence(res.begin(), res.end(), desired);
     }
 
     Image img;
@@ -1188,6 +1661,14 @@ struct RegionGrowingTest
                     shouldEqual(dist, desired);
             }
         }
+
+        vigra::IImage wres(img.size());
+
+        watershedsRegionGrowing(srcImageRange(img), destImage(wres),
+                                WatershedOptions().completeGrow()
+                                 .seedOptions(SeedOptions().minima().threshold(1.0)));
+
+        shouldEqualSequence(res.begin(), res.end(), wres.begin());
     }
 
     void voronoiWithBorderTest()
@@ -1709,14 +2190,25 @@ struct SimpleAnalysisTestSuite
         add( testCase( &DistanceTransformTest::distanceTransformL1Test));
         add( testCase( &DistanceTransformTest::distanceTransformL2Test));
         add( testCase( &DistanceTransformTest::distanceTransformLInfTest));
+
         add( testCase( &LocalMinMaxTest::localMinimumTest));
         add( testCase( &LocalMinMaxTest::localMinimum4Test));
+        add( testCase( &LocalMinMaxTest::localMinimumTestThr));
         add( testCase( &LocalMinMaxTest::localMaximumTest));
         add( testCase( &LocalMinMaxTest::localMaximum4Test));
+        add( testCase( &LocalMinMaxTest::localMaximumTestThr));
         add( testCase( &LocalMinMaxTest::extendedLocalMinimumTest));
         add( testCase( &LocalMinMaxTest::extendedLocalMinimum4Test));
         add( testCase( &LocalMinMaxTest::extendedLocalMaximumTest));
         add( testCase( &LocalMinMaxTest::extendedLocalMaximum4Test));
+
+        add( testCase( &LocalMinMaxTest::extendedLocalMaximum3DTest));
+        add( testCase( &LocalMinMaxTest::extendedLocalMinimum3DTest));
+        add( testCase( &LocalMinMaxTest::extendedLocalMaximum3DTest2));
+        add( testCase( &LocalMinMaxTest::extendedLocalMinimum3DTest2));
+        add( testCase( &LocalMinMaxTest::localMaximum3DTest));
+        add( testCase( &LocalMinMaxTest::localMinimum3DTest));
+
         add( testCase( &LocalMinMaxTest::plateauWithHolesTest));
         add( testCase( &WatershedsTest::watershedsTest));
         add( testCase( &WatershedsTest::watersheds4Test));
