@@ -44,10 +44,13 @@ import vigra.ufunc
 
 try:
     from VigraQt import OverlayViewer, ImageCursor
-except:
+except Exception, e:
     vigra._fallbackModule('VigraQt',
-    '''    It can be obtained at
-    http://kogs-www.informatik.uni-hamburg.de/~meine/software/vigraqt/.''')
+    '''
+    %s
+    
+    If VigraQt is missing on your system, you can download it from
+    http://kogs-www.informatik.uni-hamburg.de/~meine/software/vigraqt/.''' % str(e))
     from VigraQt import OverlayViewer, ImageCursor
 
 import quickdialog
@@ -335,7 +338,7 @@ class CaptionImageViewer(qt.QFrame):
         self._captionCoords = 0,0
         self._xplaces = int(math.log10(self.viewer.image.width) + 1.0)
         self._yplaces = int(math.log10(self.viewer.image.height) + 1.0)
-        self._valueplaces = self.viewer.image.bands()*5
+        self._valueplaces = self.viewer.image.channels*5
         
         self.label = qt.QLabel(self)
         font = qt.QFont()
