@@ -178,9 +178,11 @@ struct MultiArrayShapeConverterTraits
 {
     typedef TinyVector<T, N> ShapeType;
 
-    static ShapeType * construct(void* const storage, PyObject *)
+    static void construct(void* const storage, PyObject * obj)
     {
-        return new (storage) ShapeType();
+        ShapeType * shape = new (storage) ShapeType();
+        for(int i=0; i<PySequence_Length(obj); ++i)
+            (*shape)[i] = python::extract<T>(PySequence_ITEM(obj, i));
     }
 };
 
@@ -189,9 +191,14 @@ struct MultiArrayShapeConverterTraits<0, T>
 {
     typedef ArrayVector<T> ShapeType;
 
-    static ShapeType * construct(void* const storage, PyObject * obj)
+    static void construct(void* const storage, PyObject * obj)
     {
-        return new (storage) ShapeType(PySequence_Length(obj));
+        int len = (obj == Py_None)
+                           ? 0
+                           : PySequence_Length(obj);
+        ShapeType * shape = new (storage) ShapeType(len);
+        for(int i=0; i<len; ++i)
+            (*shape)[i] = python::extract<T>(PySequence_ITEM(obj, i));
     }
 };
 
@@ -213,7 +220,11 @@ struct MultiArrayShapeConverter
         
     static void* convertible(PyObject* obj)
     {
-        if(obj == 0 || !PySequence_Check(obj) || (M != 0 && PySequence_Length(obj) != M))
+        if(obj == 0)
+            return 0;
+        if(M == 0 && obj == Py_None)
+            return obj;
+        if(!PySequence_Check(obj) || (M != 0 && PySequence_Length(obj) != M))
             return 0;
         for(int i=0; i<PySequence_Length(obj); ++i)
             if(!PyNumber_Check(PySequence_ITEM(obj, i)))
@@ -228,9 +239,7 @@ struct MultiArrayShapeConverter
         void* const storage =   
             ((python::converter::rvalue_from_python_storage<ShapeType>* ) data)->storage.bytes;
 
-        ShapeType * shape = detail::MultiArrayShapeConverterTraits<M, T>::construct(storage, obj);
-        for(int i=0; i<PySequence_Length(obj); ++i)
-            (*shape)[i] = python::extract<T>(PySequence_ITEM(obj, i));
+        detail::MultiArrayShapeConverterTraits<M, T>::construct(storage, obj);
         data->convertible = storage;
     }
 
@@ -245,8 +254,8 @@ python_ptr point2DToPythonTuple(Point2D const & point)
 {
     python_ptr tuple(PyTuple_New(2), python_ptr::keep_count);
     pythonToCppException(tuple);
-    PyTuple_SET_ITEM((PyTupleObject *)tuple.get(), 0 ,pythonFromNumber(point.x).release());
-    PyTuple_SET_ITEM((PyTupleObject *)tuple.get(), 1 ,pythonFromNumber(point.y).release());
+    PyTuple_SET_ITEM((PyTupleObject *)tuple.get(), 0 ,pythonFromData(point.x).release());
+    PyTuple_SET_ITEM((PyTupleObject *)tuple.get(), 1 ,pythonFromData(point.y).release());
     return tuple;
 }
 
@@ -313,21 +322,7 @@ void registerNumpyShapeConvertersAllTypes()
         MultiArrayShapeConverter<0, npy_intp>();
 }
 
-std::set<std::string> & exportedArrayKeys()
-{
-    static std::set<std::string> keys;
-    return keys;
-}
-
-python::list listExportedArrayKeys()
-{
-    python::list res;
-    std::set<std::string>::iterator i = exportedArrayKeys().begin();
-    for(; i != exportedArrayKeys().end(); ++i)
-        res.append(*i);
-    return res;
-}
-
+#if 0 // FIXME: reimplement to replace the Python versions for consistence?
 PyObject * 
 constructNumpyArrayFromShape(python::object type, ArrayVector<npy_intp> const & shape, 
                        unsigned int spatialDimensions, unsigned int channels,
@@ -356,6 +351,25 @@ constructNumpyArrayFromArray(python::object type, NumpyAnyArray array,
     }
     return res;
 }
+#endif
+
+PyObject * 
+constructArrayFromAxistags(python::object type, ArrayVector<npy_intp> const & shape, 
+                           NPY_TYPES typeCode, AxisTags const & axistags, bool init)
+{
+    PyAxisTags pyaxistags(python_ptr(python::object(axistags).ptr()));
+    
+    ArrayVector<npy_intp> norm_shape(shape);
+    if(pyaxistags.size() > 0)
+    {
+        ArrayVector<npy_intp> permutation(pyaxistags.permutationToNormalOrder());
+        applyPermutation(permutation.begin(), permutation.end(), shape.begin(), norm_shape.begin());
+    }
+    
+    TaggedShape tagged_shape(norm_shape, pyaxistags);
+    // FIXME: check that type is an array class?
+    return constructArray(tagged_shape, typeCode, init, python_ptr(type.ptr()));
+}
 
 void registerNumpyArrayConverters()
 {
@@ -365,48 +379,10 @@ void registerNumpyArrayConverters()
     NumpyAnyArrayConverter();
     
     python::docstring_options doc_options(true, true, false);
-    
-    python::def("registerPythonArrayType", &detail::registerPythonArrayType, 
-             (python::arg("key"), python::arg("typeobj"), python::arg("typecheck") = python::object()), 
-             "registerPythonArrayType(key, typeobj, typecheck = None)\n\n"
-             "Register a mapping from a C++ type (identified by its string 'key') to a\n"
-             "Python-defined array type 'typeobj'. This mapping is applied whenever an\n"
-             "object of this C++ type is contructed or returned to Python. The registered\n"
-             "'typeobj' must be a subclass of numpy.ndarray.\n\n"
-             "'key' can be a fully qualified type (e.g. 'NumpyArray<2, RGBValue<float32> >'),\n"
-             "or it can contain '*' as a placeholder for the value type (e.g.\n"
-             "'NumpyArray<2, RGBValue<*> >'). The fully qualified key takes precedence over\n"
-             "the placeholder key when both have been registered. If no key was registered\n"
-             "for a particular C++ type, it is always handled as a plain numpy ndarray. Call\n"
-             "'listExportedArrayKeys()' for the list of recognized keys.\n\n"
-             "Optionally, you can pass a 'typecheck' function. This function is executed when\n"
-             "an instance of 'typeobj' is passed to C++ in order to find out whether\n"
-             "conversion into the C++ type identified by 'key' is allowed. The function must\n"
-             "return 'True' or 'False'. This functionality is useful to distinguish object\n"
-             "(e.g. during overload resolution) that have identical memory layout, but\n"
-             "different semantics, such as a multiband image (two spatial dimensions and\n"
-             "one spectral dimension) vs. a singleband volume (three spatial dimensions).\n\n"
-             "Usage (see vigra/arraytypes.py for a more realistic example)::\n"
-             "\n"
-             "   class Image(numpy.ndarray):\n"
-             "      spatialDimensions = 2\n"
-             "   class Volume(numpy.ndarray):\n"
-             "      spatialDimensions = 3\n\n"
-             "   def checkImage(obj):\n"
-             "      return obj.spatialDimensions == 2\n"
-             "   def checkVolume(obj):\n"
-             "      return obj.spatialDimensions == 3\n\n"
-             "   registerPythonArrayType('NumpyArray<2, RGBValue<*> >', Image, checkImage)\n"
-             "   registerPythonArrayType('NumpyArray<3, Singleband<*> >', Volume, checkVolume)\n"
-             "\n"
-             "The current mapping configuration can be obtained by calling :func:`~vigra.listExportedArrayKeys`.\n\n");
-    python::def("listExportedArrayKeys", &listExportedArrayKeys,
-        "List the currently active type mappings between C++ NumpyArray and Python array "
-        "types.  This provides status information for :func:`~vigra.registerPythonArrayType`.\n\n");
-    
+        
     doc_options.disable_all();
-    python::def("constructNumpyArray", &constructNumpyArrayFromShape);
-    python::def("constructNumpyArray", &constructNumpyArrayFromArray);
+    python::def("constructArrayFromAxistags", &constructArrayFromAxistags);
+    // python::def("constructNumpyArray", &constructNumpyArrayFromArray);
 }
 
 } // namespace vigra
