@@ -46,6 +46,7 @@
 #include <vigra/recursiveconvolution.hxx>
 #include <vigra/nonlineardiffusion.hxx>
 #include "vigranumpykernel.hxx"
+#include "vigranumpyscaleparam.hxx"
 
 namespace python = boost::python;
 
@@ -54,7 +55,7 @@ namespace vigra
 
 template < class VoxelType, unsigned int ndim >
 NumpyAnyArray 
-pythonConvolveOneDimensionND(NumpyArray<ndim, Multiband<VoxelType> > volume,
+pythonConvolveOneDimensionND(NumpyArray<ndim, Multiband<VoxelType> > array,
                              unsigned int dim,
                              Kernel const & kernel,
                              NumpyArray<ndim, Multiband<VoxelType> > res=python::object())
@@ -62,58 +63,70 @@ pythonConvolveOneDimensionND(NumpyArray<ndim, Multiband<VoxelType> > volume,
     vigra_precondition(dim < ndim-1,
            "convolveOneDimension(): dim out of range.");
 
-    res.reshapeIfEmpty(volume.shape(), "convolveOneDimension(): Output array has wrong shape.");
+    res.reshapeIfEmpty(array.taggedShape(), 
+            "convolveOneDimension(): Output array has wrong shape.");
     
-    for(int k=0;k<volume.shape(ndim-1);++k)
-	{
-    	MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bvolume = volume.bindOuter(k);
-    	MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bres = res.bindOuter(k);
-    	convolveMultiArrayOneDimension(srcMultiArrayRange(bvolume), destMultiArray(bres), dim, kernel);
-	}
-    return res;
-}
-
-template < class VoxelType, unsigned int ndim >
-NumpyAnyArray 
-pythonSeparableConvolveND_1Kernel(NumpyArray<ndim, Multiband<VoxelType> > volume,
-                                  Kernel const & kernel,
-                                  NumpyArray<ndim, Multiband<VoxelType> > res=python::object())
-{
-    res.reshapeIfEmpty(volume.shape(), "convolve(): Output array has wrong shape.");
-    
-    for(int k=0;k<volume.shape(ndim-1);++k)
+    PyAllowThreads _pythread;
+    for(int k=0;k<array.shape(ndim-1);++k)
     {
-    	MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bvolume = volume.bindOuter(k);
-    	MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bres = res.bindOuter(k);
-    	separableConvolveMultiArray(srcMultiArrayRange(bvolume), destMultiArray(bres), kernel);
+        MultiArrayView<ndim-1, VoxelType, StridedArrayTag> barray = array.bindOuter(k);
+        MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bres = res.bindOuter(k);
+        convolveMultiArrayOneDimension(srcMultiArrayRange(barray), 
+                                       destMultiArray(bres), dim, kernel);
     }
     return res;
 }
 
 template < class VoxelType, unsigned int ndim >
 NumpyAnyArray 
-pythonSeparableConvolveND_NKernels(NumpyArray<ndim, Multiband<VoxelType> > volume,
+pythonSeparableConvolveND_1Kernel(NumpyArray<ndim, Multiband<VoxelType> > array,
+                                  Kernel const & kernel,
+                                  NumpyArray<ndim, Multiband<VoxelType> > res=python::object())
+{
+    res.reshapeIfEmpty(array.taggedShape(), 
+            "convolve(): Output array has wrong shape.");
+    
+    PyAllowThreads _pythread;
+    for(int k=0;k<array.shape(ndim-1);++k)
+    {
+        MultiArrayView<ndim-1, VoxelType, StridedArrayTag> barray = array.bindOuter(k);
+        MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bres = res.bindOuter(k);
+        separableConvolveMultiArray(srcMultiArrayRange(barray), destMultiArray(bres), kernel);
+    }
+    return res;
+}
+
+template < class VoxelType, unsigned int ndim >
+NumpyAnyArray 
+pythonSeparableConvolveND_NKernels(NumpyArray<ndim, Multiband<VoxelType> > array,
                                    python::tuple pykernels,
                                    NumpyArray<ndim, Multiband<VoxelType> > res=python::object())
 {
     if(python::len(pykernels) == 1)
-        return pythonSeparableConvolveND_1Kernel(volume, 
-                              python::extract<Kernel1D<KernelValueType> const &>(pykernels[0]), res);
-        
+    {
+        return pythonSeparableConvolveND_1Kernel(array, 
+                    python::extract<Kernel1D<KernelValueType> const &>(pykernels[0]), res);
+    }
+    
     vigra_precondition(python::len(pykernels) == ndim-1,
        "convolve(): Number of kernels must be 1 or equal to the number of spatial dimensions.");
        
-    res.reshapeIfEmpty(volume.shape(), "convolve(): Output array has wrong shape.");
-    
     ArrayVector<Kernel1D<KernelValueType> > kernels;
     for(unsigned int k=0; k < ndim-1; ++k)
         kernels.push_back(python::extract<Kernel1D<KernelValueType> const &>(pykernels[k]));
+        
+    kernels = array.permuteLikewise(kernels);
 
-    for(int k=0; k < volume.shape(ndim-1); ++k)
+    res.reshapeIfEmpty(array.taggedShape(), 
+            "convolve(): Output array has wrong shape.");
+    
+    PyAllowThreads _pythread;
+    for(int k=0; k < array.shape(ndim-1); ++k)
     {
-    	MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bvolume = volume.bindOuter(k);
-    	MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bres = res.bindOuter(k);
-    	separableConvolveMultiArray(srcMultiArrayRange(bvolume), destMultiArray(bres), kernels.begin());
+        MultiArrayView<ndim-1, VoxelType, StridedArrayTag> barray = array.bindOuter(k);
+        MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bres = res.bindOuter(k);
+        separableConvolveMultiArray(srcMultiArrayRange(barray), 
+                                    destMultiArray(bres), kernels.begin());
     }
     return res;
 }
@@ -124,15 +137,17 @@ pythonConvolveImage(NumpyArray<3, Multiband<PixelType> > image,
                     TwoDKernel const & kernel, 
                     NumpyArray<3, Multiband<PixelType> > res = python::object())
 {
-	res.reshapeIfEmpty(image.shape(), "convolve(): Output array has wrong shape.");
+    res.reshapeIfEmpty(image.taggedShape(), 
+            "convolve(): Output array has wrong shape.");
 
-	for(int k=0;k<image.shape(2);++k)
-	{
-	    MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
-	    MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
-	    convolveImage(srcImageRange(bimage), destImage(bres),
-	                  kernel2d(kernel));
-	}
+    PyAllowThreads _pythread;
+    for(int k=0;k<image.shape(2);++k)
+    {
+        MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
+        MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
+        convolveImage(srcImageRange(bimage), destImage(bres),
+                      kernel2d(kernel));
+    }
     return res;
 }
 
@@ -148,62 +163,63 @@ pythonNormalizedConvolveImage(NumpyArray<3, Multiband<PixelType> > image,
     vigra_precondition(mask.shape(0)==image.shape(0) && mask.shape(1)==image.shape(1),
                "normalizedConvolveImage(): mask dimensions must be same as image dimensions");
 
-    res.reshapeIfEmpty(image.shape(), "normalizedConvolveImage(): Output array has wrong shape.");
+    res.reshapeIfEmpty(image.taggedShape(), 
+           "normalizedConvolveImage(): Output array has wrong shape.");
 
-	for(int k=0;k<image.shape(2);++k)
-	{
-	    MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
-	    MultiArrayView<2, PixelType, StridedArrayTag> bmask = mask.bindOuter(mask.shape(2)==1?0:k);
-	    MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
-	    normalizedConvolveImage(srcImageRange(bimage), srcImage(bmask), destImage(bres),
-	                            kernel2d(kernel));
-	}
+    PyAllowThreads _pythread;
+    for(int k=0;k<image.shape(2);++k)
+    {
+        MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
+        MultiArrayView<2, PixelType, StridedArrayTag> bmask = mask.bindOuter(mask.shape(2)==1?0:k);
+        MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
+        normalizedConvolveImage(srcImageRange(bimage), srcImage(bmask), destImage(bres),
+                                kernel2d(kernel));
+    }
     return res;
 }
 
 template < class VoxelType, unsigned int ndim >
 NumpyAnyArray 
-pythonGaussianSmoothing(NumpyArray<ndim, Multiband<VoxelType> > volume,
-                        python::tuple sigmas,
-                        NumpyArray<ndim, Multiband<VoxelType> > res=python::object())
+pythonGaussianSmoothing(NumpyArray<ndim, Multiband<VoxelType> > array,
+                        python::object sigma,
+                        NumpyArray<ndim, Multiband<VoxelType> > res=python::object(),
+                        python::object sigma_d = python::object(0.0), 
+                        python::object step_size = python::object(1.0),
+                        double window_size = 0.0, 
+                        python::object roi = python::object())
 {
-    unsigned int sigmaCount = python::len(sigmas);
-    vigra_precondition(sigmaCount == 1 || sigmaCount == ndim-1,
-       "gaussianSmoothing(): Number of kernels must be 1 or equal to the number of spatial dimensions.");
-       
-    ArrayVector<Kernel1D<KernelValueType> > kernels;
-    for(unsigned int k=0; k < sigmaCount; ++k)
+    static const unsigned int N = ndim - 1;
+    
+    pythonScaleParam<N> params(sigma, sigma_d, step_size, "gaussianSmoothing");
+    
+    params.permuteLikewise(array);
+    
+    ConvolutionOptions<N> opt(params().filterWindowSize(window_size));
+    
+    if(roi != python::object())
     {
-        KernelValueType sigma = python::extract<KernelValueType>(sigmas[k]);        
-        kernels.push_back(Kernel1D<KernelValueType>());
-        kernels.back().initGaussian(sigma);
+        typedef typename MultiArrayShape<N>::type Shape;
+        Shape start = array.permuteLikewise(python::extract<Shape>(roi[0])());
+        Shape stop  = array.permuteLikewise(python::extract<Shape>(roi[1])());
+        opt.subarray(start, stop);
+        res.reshapeIfEmpty(array.taggedShape().resize(stop-start), 
+                "gaussianSmoothing(): Output array has wrong shape.");
     }
-    for(unsigned int k=sigmaCount; k < ndim-1; ++k)
+    else
     {
-        kernels.push_back(Kernel1D<KernelValueType>(kernels.back()));
+        res.reshapeIfEmpty(array.taggedShape(), 
+                "gaussianSmoothing(): Output array has wrong shape.");
     }
     
-    res.reshapeIfEmpty(volume.shape(), "gaussianSmoothing(): Output array has wrong shape.");
-
-	Py_BEGIN_ALLOW_THREADS
-    for(int k=0;k<volume.shape(ndim-1);++k)
+    PyAllowThreads _pythread;
+    for(int k=0; k<array.shape(ndim-1); ++k)
     {
-    	MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bvolume = volume.bindOuter(k);
-    	MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bres = res.bindOuter(k);
-    	separableConvolveMultiArray(srcMultiArrayRange(bvolume), destMultiArray(bres), kernels.begin());
+        MultiArrayView<ndim-1, VoxelType, StridedArrayTag> barray = array.bindOuter(k);
+        MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bres = res.bindOuter(k);
+        gaussianSmoothMultiArray(srcMultiArrayRange(barray), destMultiArray(bres), opt);
     }
-	Py_END_ALLOW_THREADS
+    
     return res;
-
-}
-
-template < class VoxelType, unsigned int ndim >
-NumpyAnyArray 
-pythonGaussianSmoothingIsotropic(NumpyArray<ndim, Multiband<VoxelType> > volume,
-                                 double sigma,
-                                 NumpyArray<ndim, Multiband<VoxelType> > res=python::object())
-{
-    return pythonGaussianSmoothing(volume, python::make_tuple(sigma), res);
 }
 
 template < class VoxelType>
@@ -228,16 +244,20 @@ pythonRecursiveGaussian(NumpyArray<3, Multiband<VoxelType> > image,
     {
         scales.push_back(scales.back());
     }
+    scales = image.permuteLikewise(scales);
     
-    res.reshapeIfEmpty(image.shape(), "recursiveGaussianSmoothing(): Output array has wrong shape.");
+    res.reshapeIfEmpty(image.taggedShape(), 
+            "recursiveGaussianSmoothing(): Output array has wrong shape.");
+
+    PyAllowThreads _pythread;
     MultiArray<ndim-1, TmpType> tmp(image.bindOuter(0).shape());
 
     for(int k=0;k<image.shape(ndim-1);++k)
     {
-    	MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bimage = image.bindOuter(k);
-    	MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bres = res.bindOuter(k);
-    	recursiveGaussianFilterX(srcImageRange(bimage), destImage(tmp), scales[0]);
-    	recursiveGaussianFilterY(srcImageRange(tmp), destImage(bres), scales[1]);
+        MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bimage = image.bindOuter(k);
+        MultiArrayView<ndim-1, VoxelType, StridedArrayTag> bres = res.bindOuter(k);
+        recursiveGaussianFilterX(srcImageRange(bimage), destImage(tmp), scales[0]);
+        recursiveGaussianFilterY(srcImageRange(tmp), destImage(bres), scales[1]);
     }
     return res;
 
@@ -256,13 +276,15 @@ template <class PixelType>
 NumpyAnyArray 
 pythonSimpleSharpening2D(NumpyArray<3, Multiband<PixelType> > image, 
                          double sharpeningFactor,
-				         NumpyArray<3, Multiband<PixelType> > res=python::object() )
+                         NumpyArray<3, Multiband<PixelType> > res=python::object() )
 {
-	res.reshapeIfEmpty(image.shape(), "simpleSharpening2D(): Output array has wrong shape.");
-    
     vigra_precondition(sharpeningFactor >= 0 ,
        "simpleSharpening2D(): sharpeningFactor must be >= 0.");
        
+    res.reshapeIfEmpty(image.taggedShape(), 
+          "simpleSharpening2D(): Output array has wrong shape.");
+    
+    PyAllowThreads _pythread;
     for(int k=0;k<image.shape(2);++k)
     {
         MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
@@ -279,92 +301,132 @@ pythonGaussianSharpening2D(NumpyArray<3, Multiband<PixelType> > image,
                            double sharpeningFactor, double scale, 
                            NumpyArray<3, Multiband<PixelType> > res=python::object() )
 {
-	res.reshapeIfEmpty(image.shape(), "gaussianSharpening2D(): Output array has wrong shape.");
-    
-	vigra_precondition(sharpeningFactor >= 0 ,
+    vigra_precondition(sharpeningFactor >= 0 ,
        "gaussianSharpening2D(): sharpeningFactor must be >= 0.");
     vigra_precondition(sharpeningFactor >= 0 ,
        "gaussianSharpening2D(): scale must be >= 0.");
        
+    res.reshapeIfEmpty(image.taggedShape(), 
+             "gaussianSharpening2D(): Output array has wrong shape.");
+    
+    PyAllowThreads _pythread;
     for(int k=0;k<image.shape(2);++k)
-	{
-	    MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
-	    MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
-	    gaussianSharpening(srcImageRange(bimage), destImage(bres),
-	                       sharpeningFactor, scale);
-	}
+    {
+        MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
+        MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
+        gaussianSharpening(srcImageRange(bimage), destImage(bres),
+                           sharpeningFactor, scale);
+    }
     return res;
 }
 
 template <class PixelType, unsigned int N>
 NumpyAnyArray 
-pythonLaplacianOfGaussian(NumpyArray<N, Multiband<PixelType> > image,
-                          double scale, 
-                          NumpyArray<N, Multiband<PixelType> > res=python::object() )
+pythonLaplacianOfGaussian(NumpyArray<N, Multiband<PixelType> > array,
+                          python::object scale,
+                          NumpyArray<N, Multiband<PixelType> > res=python::object(),
+                          python::object sigma_d = python::object(0.0), 
+                          python::object step_size = python::object(1.0),
+                          double window_size = 0.0, 
+                          python::object roi = python::object())
 {
-	res.reshapeIfEmpty(image.shape(), "laplacianOfGaussian(): Output array has wrong shape.");
+    pythonScaleParam<N - 1> params(scale, sigma_d, step_size, "laplacianOfGaussian");
+    params.permuteLikewise(array);
+    
+    std::string description("channel-wise Laplacian of Gaussian, scale=");
+    description += asString(scale);
+    
+    ConvolutionOptions<N-1> opt(params().filterWindowSize(window_size));
+    
+    if(roi != python::object())
+    {
+        typedef typename MultiArrayShape<N-1>::type Shape;
+        Shape start = array.permuteLikewise(python::extract<Shape>(roi[0])());
+        Shape stop  = array.permuteLikewise(python::extract<Shape>(roi[1])());
+        opt.subarray(start, stop);
+        res.reshapeIfEmpty(array.taggedShape().resize(stop-start).setChannelDescription(description), 
+                "laplacianOfGaussian(): Output array has wrong shape.");
+    }
+    else
+    {
+        res.reshapeIfEmpty(array.taggedShape().setChannelDescription(description), 
+                "laplacianOfGaussian(): Output array has wrong shape.");
+    }
+    
+    PyAllowThreads _pythread;
+    for(int k=0; k<array.shape(N-1); ++k)
+    {
+        MultiArrayView<N-1, PixelType, StridedArrayTag> barray = array.bindOuter(k);
+        MultiArrayView<N-1, PixelType, StridedArrayTag> bres = res.bindOuter(k);
+        laplacianOfGaussianMultiArray(srcMultiArrayRange(barray), destMultiArray(bres), opt);
+    }
     
-	Py_BEGIN_ALLOW_THREADS
-	for(int k=0; k<image.shape(N-1); ++k)
-	{
-	    MultiArrayView<N-1, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
-	    MultiArrayView<N-1, PixelType, StridedArrayTag> bres = res.bindOuter(k);
-	    laplacianOfGaussianMultiArray(srcMultiArrayRange(bimage), destMultiArray(bres), scale);
-	}
-	Py_END_ALLOW_THREADS
     return res;
 }
 
 template <class PixelType>
-NumpyAnyArray pythonRecursiveFilter1(NumpyArray<3, Multiband<PixelType> > image,
-                                     double b, BorderTreatmentMode borderTreatment, 
-                                     NumpyArray<3, Multiband<PixelType> > res = python::object())
+NumpyAnyArray 
+pythonRecursiveFilter1(NumpyArray<3, Multiband<PixelType> > image,
+                       double b, BorderTreatmentMode borderTreatment, 
+                       NumpyArray<3, Multiband<PixelType> > res = python::object())
 {
-	res.reshapeIfEmpty(image.shape(), "recursiveFilter2D(): Output array has wrong shape.");
+    res.reshapeIfEmpty(image.taggedShape(), 
+            "recursiveFilter2D(): Output array has wrong shape.");
 
-	for(int k=0;k<image.shape(2);++k)
-	{
-	    MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
-	    MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
-	    recursiveFilterX(srcImageRange(bimage), destImage(bres), b, borderTreatment);
-	    recursiveFilterY(srcImageRange(bres), destImage(bres), b, borderTreatment);
-	}
+    PyAllowThreads _pythread;
+    for(int k=0;k<image.shape(2);++k)
+    {
+        MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
+        MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
+        recursiveFilterX(srcImageRange(bimage), destImage(bres), b, borderTreatment);
+        recursiveFilterY(srcImageRange(bres), destImage(bres), b, borderTreatment);
+    }
     return res;
 }
 
 template <class PixelType>
-NumpyAnyArray pythonRecursiveFilter2(NumpyArray<3, Multiband<PixelType> > image,
-                                     double b1, double b2, 
-                                     NumpyArray<3, Multiband<PixelType> > res = python::object())
+NumpyAnyArray 
+pythonRecursiveFilter2(NumpyArray<3, Multiband<PixelType> > image,
+                       double b1, double b2, 
+                       NumpyArray<3, Multiband<PixelType> > res = python::object())
 {
-	res.reshapeIfEmpty(image.shape(), "recursiveFilter2D(): Output array has wrong shape.");
+    res.reshapeIfEmpty(image.taggedShape(), 
+            "recursiveFilter2D(): Output array has wrong shape.");
 
-	for(int k=0;k<image.shape(2);++k)
-	{
-	    MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
-	    MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
-	    recursiveFilterX(srcImageRange(bimage), destImage(bres), b1, b2);
-	    recursiveFilterY(srcImageRange(bres), destImage(bres), b1, b2);
-	}
+    PyAllowThreads _pythread;
+    for(int k=0;k<image.shape(2);++k)
+    {
+        MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
+        MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
+        recursiveFilterX(srcImageRange(bimage), destImage(bres), b1, b2);
+        recursiveFilterY(srcImageRange(bres), destImage(bres), b1, b2);
+    }
     return res;
 }
 
 
 template <class PixelType>
-NumpyAnyArray pythonRecursiveSmooth(NumpyArray<3, Multiband<PixelType> > image,
-                                    double scale, BorderTreatmentMode borderTreatment, 
-                                    NumpyArray<3, Multiband<PixelType> > res = python::object())
+NumpyAnyArray 
+pythonRecursiveSmooth(NumpyArray<3, Multiband<PixelType> > image,
+                      double scale, BorderTreatmentMode borderTreatment, 
+                      NumpyArray<3, Multiband<PixelType> > res = python::object())
 {
     return pythonRecursiveFilter1(image, std::exp(-1.0/scale), borderTreatment, res);
 }
 
 template <class PixelType>
-NumpyAnyArray pythonRecursiveGradient(NumpyArray<2, Singleband<PixelType> > image,
-                                      double scale, 
-                                      NumpyArray<2, TinyVector<PixelType, 2> > res = python::object())
+NumpyAnyArray 
+pythonRecursiveGradient(NumpyArray<2, Singleband<PixelType> > image,
+                        double scale, 
+                        NumpyArray<2, TinyVector<PixelType, 2> > res = python::object())
 {
-	res.reshapeIfEmpty(image.shape(), "recursiveGradient2D(): Output array has wrong shape.");
+    std::string description("recursive gradient, scale=");
+    description += asString(scale);
+    
+    res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+            "recursiveGradient2D(): Output array has wrong shape.");
 
+    PyAllowThreads _pythread;
     VectorComponentValueAccessor<TinyVector<PixelType, 2> > band(0);
     recursiveFirstDerivativeX(srcImageRange(image), destImage(res, band), scale);
     recursiveSmoothY(srcImageRange(res, band), destImage(res, band), scale);
@@ -377,20 +439,26 @@ NumpyAnyArray pythonRecursiveGradient(NumpyArray<2, Singleband<PixelType> > imag
 }
 
 template <class PixelType>
-NumpyAnyArray pythonRecursiveLaplacian(NumpyArray<3, Multiband<PixelType> > image,
-                                     double scale, 
-                                     NumpyArray<3, Multiband<PixelType> > res = python::object())
+NumpyAnyArray 
+pythonRecursiveLaplacian(NumpyArray<3, Multiband<PixelType> > image,
+                         double scale, 
+                         NumpyArray<3, Multiband<PixelType> > res = python::object())
 {
-	using namespace vigra::functor;
-	
-	res.reshapeIfEmpty(image.shape(), "recursiveLaplacian2D(): Output array has wrong shape.");
+    using namespace vigra::functor;
+    
+    std::string description("channel-wise recursive Laplacian, scale=");
+    description += asString(scale);
+    
+    res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+            "recursiveLaplacian2D(): Output array has wrong shape.");
 
+    PyAllowThreads _pythread;
     MultiArrayShape<2>::type tmpShape(image.shape().begin());
     MultiArray<2, PixelType > tmp(tmpShape);
-	for(int k=0;k<image.shape(2);++k)
-	{
-	    MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
-	    MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
+    for(int k=0;k<image.shape(2);++k)
+    {
+        MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
+        MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
 
         recursiveSecondDerivativeX(srcImageRange(bimage), destImage(bres), scale);
         recursiveSmoothY(srcImageRange(bres), destImage(bres), scale);
@@ -399,7 +467,7 @@ NumpyAnyArray pythonRecursiveLaplacian(NumpyArray<3, Multiband<PixelType> > imag
         recursiveSecondDerivativeY(srcImageRange(tmp), destImage(tmp), scale);
         
         combineTwoImages(srcImageRange(bres), srcImage(tmp), destImage(bres), Arg1()+Arg2());
-	}
+    }
     return res;
 }
 
@@ -410,16 +478,16 @@ void defineConvolutionFunctions()
     docstring_options doc_options(true, true, false);
 
     def("convolveOneDimension",
-    	registerConverters(&pythonConvolveOneDimensionND<float,3>),
-    	(arg("image"), arg("dim"), arg("kernel"), arg("out")=python::object()),
+        registerConverters(&pythonConvolveOneDimensionND<float,3>),
+        (arg("image"), arg("dim"), arg("kernel"), arg("out")=python::object()),
         "Convolution along a single dimension of a 2D scalar or multiband image. "
         "'kernel' must be an instance of Kernel1D.\n"
         "\n"
         "For details see convolveMultiArrayOneDimension_ in the vigra C++ documentation.\n");
 
     def("convolveOneDimension",
-    	registerConverters(&pythonConvolveOneDimensionND<float,4>),
-    	(arg("volume"), arg("dim"), arg("kernel"), arg("out")=python::object()), 
+        registerConverters(&pythonConvolveOneDimensionND<float,4>),
+        (arg("volume"), arg("dim"), arg("kernel"), arg("out")=python::object()), 
         "Likewise for a 3D scalar or multiband volume.\n");
 
     def("convolve", registerConverters(&pythonSeparableConvolveND_1Kernel<float,3>),
@@ -465,31 +533,40 @@ void defineConvolutionFunctions()
         "For details, see normalizedConvolveImage_ in the C++ documentation.\n");
 
     def("gaussianSmoothing",
-        registerConverters(&pythonGaussianSmoothingIsotropic<float,3>),               
-        (arg("image"), arg("sigma"), arg("out")=python::object()),
-        "Perform Gaussian smoothing of a 2D or 3D scalar or multiband image.\n\n"
+        registerConverters(&pythonGaussianSmoothing<float,3>),
+        (arg("array"), arg("sigma"), arg("out")=python::object(), 
+         arg("sigma_d")=0.0, arg("step_size")=1.0, arg("window_size")=0.0, arg("roi")=python::object()),
+        "Perform Gaussian smoothing of a 2D or 3D scalar or multiband array.\n\n"
         "Each channel of the array is smoothed independently. "
         "If 'sigma' is a single value, an isotropic Gaussian filter at this scale is "
         "applied (i.e. each dimension is smoothed in the same way). "
-        "If 'sigma' is a tuple of values, the amount of smoothing will be different "
-        "for each spatial dimension. The length of the tuple must be equal to the "
-        "number of spatial dimensions.\n\n"        
-        "For details see gaussianSmoothing_ in the vigra C++ documentation.\n");
-
-    def("gaussianSmoothing",
-        registerConverters(&pythonGaussianSmoothing<float,3>),               
-        (arg("image"), arg("sigmas"), arg("out")=python::object()),
-        "Smooth image with an anisotropic Gaussian.\n");
-
-    def("gaussianSmoothing",
-        registerConverters(&pythonGaussianSmoothingIsotropic<float,4>),               
-        (arg("volume"), arg("sigma"), arg("out")=python::object()),
-        "Smooth volume with an isotropic Gaussian.\n");
+        "If 'sigma' is a tuple or list of values, the amount of smoothing will be different "
+        "for each spatial dimension.\n"
+        "The optional 'sigma_d' (single, tuple, or list) (single, tuple, or list) denotes the resolution standard deviation "
+        "per axis, the optional 'step_size' (single, tuple, or list) the distance between two adjacent "
+        "pixels for each dimension. "
+        "The length of the tuples or lists must be equal to the "
+        "number of spatial dimensions.\n\n"
+        "'window_size' specifies the ratio between the effective filter scale and "
+        "the size of the filter window. Use a value around 2.0 to speed-up "
+        "the computation by increasing the error resulting from cutting off the Gaussian. "
+        "For the default 0.0, the window size is automatically determined.\n"
+        "\n"
+        "If 'roi' is not None, it must specify the desired region-of-interest as "
+        "a pair '(first_point, beyond_last_point)' (e.g. 'roi=((10,20), (200,250))'). "
+        "As usual, the second point is the first point outside the ROI, and the ROI "
+        "must not be outside the input array dimensions. "
+        "The coordinates refer only to non-channel axes - if your array has an explicit "
+        "channel axis, the ROI dimension must be one less than the array dimension. "
+        "If you pass in an explicit 'out' array and specify an ROI, the 'out' array "
+        "must have the shape of the ROI.\n\n"
+        "For details see gaussianSmoothing_ and ConvolutionOptions_ in the vigra C++ documentation.\n");
 
     def("gaussianSmoothing",
         registerConverters(&pythonGaussianSmoothing<float,4>),
-        (arg("volume"), arg("sigmas"), arg("out")=python::object()),
-        "Smooth volume with an anisotropic Gaussian.\n");
+        (arg("array"), arg("sigma"), arg("out")=python::object(), 
+         arg("sigma_d")=0.0, arg("step_size")=1.0, arg("window_size")=0.0, arg("roi")=python::object()),
+        "Smooth volume with Gaussian.\n");
 
     def("recursiveGaussianSmoothing2D",
         registerConverters(&pythonRecursiveGaussian<float>),               
@@ -526,14 +603,25 @@ void defineConvolutionFunctions()
           
     def("laplacianOfGaussian", 
          registerConverters(&pythonLaplacianOfGaussian<float,3>),
-         (arg("image"), arg("scale") = 1.0, arg("out") = python::object()),
-          "Filter scalar image with the Laplacian of Gaussian operator at the given scale.\n"
-          "\n"
-          "For details see laplacianOfGaussianMultiArray_ in the vigra C++ documentation.\n");
+         (arg("array"), arg("scale") = 1.0, arg("out") = python::object(), 
+          arg("sigma_d") = 0.0, arg("step_size") = 1.0, arg("window_size")=0.0, arg("roi")=python::object()),
+          "Filter 2D or 3D scalar array with the Laplacian of Gaussian operator at the given scale.\n\n"
+          "If 'sigma' is a single value, an isotropic filter at this scale is "
+          "applied (i.e., each dimension is filtered in the same way). "
+          "If 'sigma' is a tuple or list of values, the amount of smoothing "
+          "will be different for each spatial dimension.\n"
+          "The optional 'sigma_d' (single, tuple, or list) denotes the resolution standard deviation "
+          "per axis, the optional 'step_size' (single, tuple, or list) the distance between two adjacent "
+          "pixels for each dimension. "
+          "The length of the tuples or lists must be equal to the "
+          "number of spatial dimensions.\n\n" 
+          "'window_size' and 'roi' have the same meaning as in :func:`gaussianSmoothing`.\n\n"
+          "For details see laplacianOfGaussianMultiArray_ and ConvolutionOptions_ in the vigra C++ documentation.\n");
 
     def("laplacianOfGaussian", 
          registerConverters(&pythonLaplacianOfGaussian<float,4>),
-         (arg("volume"), arg("scale") = 1.0, arg("out") = python::object()),
+         (arg("array"), arg("scale") = 1.0, arg("out") = python::object(), 
+         arg("sigma_d") = 0.0, arg("step_size") = 1.0, arg("window_size")=0.0, arg("roi")=python::object()),
          "Likewise for a scalar volume.\n");
 
     def("recursiveFilter2D", registerConverters(&pythonRecursiveFilter1<float>),
