@@ -58,13 +58,16 @@ pythonDiscRankOrderFilter(NumpyArray<3, Multiband<PixelType> > image,
         "Rank must be in the range 0.0 <= rank <= 1.0");
     vigra_precondition(radius >= 0, "Radius must be >= 0.");
 
-    res.reshapeIfEmpty(image.shape(),"discRankOrderFilter(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(image.taggedShape(),
+            "discRankOrderFilter(): Output image has wrong dimensions");
 
-    for(int k=0;k<image.shape(2);++k)
+    PyAllowThreads _pythread;
+    for(int k=0; k<image.shape(2); ++k)
     { 
         MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
         MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
-        discRankOrderFilter(srcImageRange(bimage,StandardValueAccessor<UInt8>()), destImage(bres), radius, rank);
+        discRankOrderFilter(srcImageRange(bimage,StandardValueAccessor<UInt8>()), 
+                            destImage(bres), radius, rank);
     }
     return res;
 }
@@ -84,14 +87,17 @@ pythonDiscRankOrderFilterWithMask(NumpyArray<3, Multiband<PixelType> > image,
     vigra_precondition(mask.shape(0)==image.shape(0) && mask.shape(1)==image.shape(1),
                "discRankOrderFilterWithMaks(): mask dimensions must be same as image dimensions");
 
-    res.reshapeIfEmpty(image.shape(),"discRankOrderFilterWithMask(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(image.taggedShape(),
+            "discRankOrderFilterWithMask(): Output image has wrong dimensions");
 
-    for(int k=0;k<image.shape(2);++k)
+    PyAllowThreads _pythread;
+    for(int k=0; k<image.shape(2); ++k)
     { 
         MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
         MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
         MultiArrayView<2, PixelType, StridedArrayTag> bmask = mask.bindOuter(mask.shape(2)==1?0:k);
-        discRankOrderFilterWithMask(srcImageRange(bimage,StandardValueAccessor<UInt8>()), srcImage(bmask),
+        discRankOrderFilterWithMask(srcImageRange(bimage,StandardValueAccessor<UInt8>()), 
+                                    srcImage(bmask),
                                     destImage(bres), radius, rank);
     }
 
@@ -133,11 +139,13 @@ pythonDiscOpening(NumpyArray<3, Multiband<PixelType> > image,
 {
     vigra_precondition(radius >= 0, "Radius must be >=0.");
 
-    res.reshapeIfEmpty(image.shape(),"discOpening(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(image.taggedShape(),
+            "discOpening(): Output image has wrong dimensions");
 
-    MultiArray<2,PixelType> tmp(MultiArrayShape<2>::type(image.shape(0),image.shape(1)));
+    PyAllowThreads _pythread;
+    MultiArray<2,PixelType> tmp(MultiArrayShape<2>::type(image.shape(0), image.shape(1)));
 
-    for(int k=0;k<image.shape(2);++k)
+    for(int k=0; k<image.shape(2); ++k)
     {
         MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
         MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
@@ -155,11 +163,13 @@ pythonDiscClosing(NumpyArray<3, Multiband<PixelType> > image,
 {
     vigra_precondition(radius >= 0, "Radius must be >=0.");
 
-    res.reshapeIfEmpty(image.shape(),"discClosing(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(image.taggedShape(),
+            "discClosing(): Output image has wrong dimensions");
 
-    MultiArray<2,PixelType> tmp(MultiArrayShape<2>::type(image.shape(0),image.shape(1)));
+    PyAllowThreads _pythread;
+    MultiArray<2,PixelType> tmp(MultiArrayShape<2>::type(image.shape(0), image.shape(1)));
 
-    for(int k=0;k<image.shape(2);++k)
+    for(int k=0; k<image.shape(2); ++k)
     {
         MultiArrayView<2, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
         MultiArrayView<2, PixelType, StridedArrayTag> bres = res.bindOuter(k);
@@ -171,171 +181,217 @@ pythonDiscClosing(NumpyArray<3, Multiband<PixelType> > image,
 
 template < int dim, class PixelType >
 NumpyAnyArray 
-pythonMultiBinaryErosion(NumpyArray<dim, Multiband<PixelType> > image,
+pythonMultiBinaryErosion(NumpyArray<dim, Multiband<PixelType> > array,
                          double radius, 
                          NumpyArray<dim, Multiband<PixelType> > res)
 {
-    res.reshapeIfEmpty(image.shape(),"multiBinaryErosion(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(array.taggedShape(),
+            "multiBinaryErosion(): Output image has wrong dimensions");
 
-    for(int k=0;k<image.shape(dim-1);++k)
+    PyAllowThreads _pythread;
+    for(int k=0; k<array.shape(dim-1); ++k)
     {
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bres=res.bindOuter(k);
-        multiBinaryErosion(srcMultiArrayRange(bimage),destMultiArray(bres), radius);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> barray = array.bindOuter(k);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> bres = res.bindOuter(k);
+        multiBinaryErosion(srcMultiArrayRange(barray), destMultiArray(bres), radius);
     }
     return res;
 }
+
 template < int dim, class PixelType >
 NumpyAnyArray 
-pythonMultiBinaryDilation(NumpyArray<dim, Multiband<PixelType> > image,
+pythonMultiBinaryDilation(NumpyArray<dim, Multiband<PixelType> > array,
                           double radius, 
                           NumpyArray<dim, Multiband<PixelType> > res)
 {
-    res.reshapeIfEmpty(image.shape(),"multiBinaryDilation(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(array.taggedShape(),
+            "multiBinaryDilation(): Output image has wrong dimensions");
 
-    for(int k=0;k<image.shape(dim-1);++k)
+    PyAllowThreads _pythread;
+    for(int k=0; k<array.shape(dim-1); ++k)
     {
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bres=res.bindOuter(k);
-        multiBinaryDilation(srcMultiArrayRange(bimage),destMultiArray(bres), radius);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> barray = array.bindOuter(k);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> bres = res.bindOuter(k);
+        multiBinaryDilation(srcMultiArrayRange(barray), destMultiArray(bres), radius);
     }
     return res;
 }
 
 template <int dim, class PixelType >
 NumpyAnyArray 
-pythonMultiBinaryOpening(NumpyArray<dim, Multiband<PixelType> > image, 
+pythonMultiBinaryOpening(NumpyArray<dim, Multiband<PixelType> > array, 
                          double radius, 
                          NumpyArray<dim, Multiband<PixelType> > res)
 {
-    res.reshapeIfEmpty(image.shape(),"multiBinaryOpening(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(array.taggedShape(),
+            "multiBinaryOpening(): Output image has wrong dimensions");
 
-    MultiArray<dim-1,PixelType> tmp(typename MultiArrayShape<dim-1>::type(image.shape().begin()));
+    PyAllowThreads _pythread;
+    MultiArray<dim-1,PixelType> tmp(typename MultiArrayShape<dim-1>::type(array.shape().begin()));
 
-    for(int k=0;k<image.shape(dim-1);++k)
+    for(int k=0; k<array.shape(dim-1); ++k)
     {
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> barray = array.bindOuter(k);
         MultiArrayView<dim-1, PixelType, StridedArrayTag> bres = res.bindOuter(k);
-        multiBinaryErosion(srcMultiArrayRange(bimage),destMultiArray(tmp), radius);
-        multiBinaryDilation(srcMultiArrayRange(tmp),destMultiArray(bres), radius);
+        multiBinaryErosion(srcMultiArrayRange(barray), destMultiArray(tmp), radius);
+        multiBinaryDilation(srcMultiArrayRange(tmp), destMultiArray(bres), radius);
     }
     return res;
 }
 
 template <int dim, class PixelType >
 NumpyAnyArray 
-pythonMultiBinaryClosing(NumpyArray<dim, Multiband<PixelType> > image, 
+pythonMultiBinaryClosing(NumpyArray<dim, Multiband<PixelType> > array, 
                          double radius, 
                          NumpyArray<dim, Multiband<PixelType> > res)
 {
-    res.reshapeIfEmpty(image.shape(),"multiBinaryOpening(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(array.taggedShape(),
+            "multiBinaryOpening(): Output image has wrong dimensions");
 
-    MultiArray<dim-1,PixelType> tmp(typename MultiArrayShape<dim-1>::type(image.shape().begin()));
+    PyAllowThreads _pythread;
+    MultiArray<dim-1,PixelType> tmp(typename MultiArrayShape<dim-1>::type(array.shape().begin()));
 
-    for(int k=0;k<image.shape(dim-1);++k)
+    for(int k=0; k<array.shape(dim-1); ++k)
     {
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> barray = array.bindOuter(k);
         MultiArrayView<dim-1, PixelType, StridedArrayTag> bres = res.bindOuter(k);
-        multiBinaryDilation(srcMultiArrayRange(bimage),destMultiArray(tmp), radius);
-        multiBinaryErosion(srcMultiArrayRange(tmp),destMultiArray(bres), radius);
+        multiBinaryDilation(srcMultiArrayRange(barray), destMultiArray(tmp), radius);
+        multiBinaryErosion(srcMultiArrayRange(tmp), destMultiArray(bres), radius);
     }
     return res;
 }
 
 template < int dim , class PixelType>
 NumpyAnyArray 
-pythonMultiGrayscaleErosion(NumpyArray<dim, Multiband<PixelType> > image, 
+pythonMultiGrayscaleErosion(NumpyArray<dim, Multiband<PixelType> > array, 
                             double sigma, 
                             NumpyArray<dim, Multiband<PixelType> > res)
 {
-    res.reshapeIfEmpty(image.shape(),"multiGrayscaleErosion(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(array.taggedShape(),
+            "multiGrayscaleErosion(): Output image has wrong dimensions");
 
-    for(int k=0;k<image.shape(dim-1);++k)
+    PyAllowThreads _pythread;
+    for(int k=0; k<array.shape(dim-1); ++k)
     {
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bres=res.bindOuter(k);
-        multiGrayscaleErosion(srcMultiArrayRange(bimage),destMultiArray(bres), sigma);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> barray = array.bindOuter(k);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> bres = res.bindOuter(k);
+        multiGrayscaleErosion(srcMultiArrayRange(barray), destMultiArray(bres), sigma);
     }
     return res;
 }
 template < int dim, class PixelType >
 NumpyAnyArray 
-pythonMultiGrayscaleDilation(NumpyArray<dim, Multiband<PixelType> > image, 
+pythonMultiGrayscaleDilation(NumpyArray<dim, Multiband<PixelType> > array, 
                              double sigma, 
                              NumpyArray<dim, Multiband<PixelType> > res)
 {
-    res.reshapeIfEmpty(image.shape(),"multiGrayscaleDilation(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(array.taggedShape(),
+            "multiGrayscaleDilation(): Output image has wrong dimensions");
 
-    for(int k=0;k<image.shape(dim-1);++k)
+    PyAllowThreads _pythread;
+    for(int k=0; k<array.shape(dim-1); ++k)
     {
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bres=res.bindOuter(k);
-        multiGrayscaleDilation(srcMultiArrayRange(bimage),destMultiArray(bres), sigma);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> barray = array.bindOuter(k);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> bres = res.bindOuter(k);
+        multiGrayscaleDilation(srcMultiArrayRange(barray), destMultiArray(bres), sigma);
     }
     return res;
 }
 
 template <int dim, class PixelType>
 NumpyAnyArray 
-pythonMultiGrayscaleOpening(NumpyArray<dim, Multiband<PixelType> > image, 
+pythonMultiGrayscaleOpening(NumpyArray<dim, Multiband<PixelType> > array, 
                             double sigma, 
                             NumpyArray<dim, Multiband<PixelType> > res)
 {
-    res.reshapeIfEmpty(image.shape(),"multiGrayscaleOpening(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(array.taggedShape(),
+            "multiGrayscaleOpening(): Output image has wrong dimensions");
 
-    MultiArray<dim-1,PixelType> tmp(typename MultiArrayShape<dim-1>::type(image.shape().begin()));
+    PyAllowThreads _pythread;
+    MultiArray<dim-1,PixelType> tmp(typename MultiArrayShape<dim-1>::type(array.shape().begin()));
 
-    for(int k=0;k<image.shape(dim-1);++k)
+    for(int k=0; k<array.shape(dim-1); ++k)
     {
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> barray = array.bindOuter(k);
         MultiArrayView<dim-1, PixelType, StridedArrayTag> bres = res.bindOuter(k);
-        multiGrayscaleErosion(srcMultiArrayRange(bimage),destMultiArray(tmp), sigma);
-        multiGrayscaleDilation(srcMultiArrayRange(tmp),destMultiArray(bres), sigma);
+        multiGrayscaleErosion(srcMultiArrayRange(barray), destMultiArray(tmp), sigma);
+        multiGrayscaleDilation(srcMultiArrayRange(tmp), destMultiArray(bres), sigma);
     }
     return res;
 }
 
 template <int dim, class PixelType>
 NumpyAnyArray 
-pythonMultiGrayscaleClosing(NumpyArray<dim, Multiband<PixelType> > image, 
+pythonMultiGrayscaleClosing(NumpyArray<dim, Multiband<PixelType> > array, 
                             double sigma, 
                             NumpyArray<dim, Multiband<PixelType> > res)
 {
-    res.reshapeIfEmpty(image.shape(),"multiGrayscaleClosing(): Output image has wrong dimensions");
+    res.reshapeIfEmpty(array.taggedShape(),
+            "multiGrayscaleClosing(): Output image has wrong dimensions");
 
-    MultiArray<dim-1,PixelType> tmp(typename MultiArrayShape<dim-1>::type(image.shape().begin()));
+    PyAllowThreads _pythread;
+    MultiArray<dim-1,PixelType> tmp(typename MultiArrayShape<dim-1>::type(array.shape().begin()));
 
-    for(int k=0;k<image.shape(dim-1);++k)
+    for(int k=0; k<array.shape(dim-1); ++k)
     {
-        MultiArrayView<dim-1, PixelType, StridedArrayTag> bimage = image.bindOuter(k);
+        MultiArrayView<dim-1, PixelType, StridedArrayTag> barray = array.bindOuter(k);
         MultiArrayView<dim-1, PixelType, StridedArrayTag> bres = res.bindOuter(k);
-        multiGrayscaleDilation(srcMultiArrayRange(bimage),destMultiArray(tmp), sigma);
-        multiGrayscaleErosion(srcMultiArrayRange(tmp),destMultiArray(bres), sigma);
+        multiGrayscaleDilation(srcMultiArrayRange(barray), destMultiArray(tmp), sigma);
+        multiGrayscaleErosion(srcMultiArrayRange(tmp), destMultiArray(bres), sigma);
     }
     return res;
 }
 
+namespace detail {
+
+template <class PixelType>
+struct IsBackgroundAccessor
+{
+    typedef bool value_type;
+    
+    template <class Iterator>
+    value_type operator()(Iterator const & i) const
+    {
+        return *i == NumericTraits<PixelType>::zero();
+    }
+};
+
+} // namespace detail
+
 template < class PixelType, typename DestPixelType >
 NumpyAnyArray 
 pythonDistanceTransform2D(NumpyArray<2, Singleband<PixelType> > image,
-                          PixelType background, 
+                          bool background, 
                           int norm,
                           NumpyArray<2, Singleband<DestPixelType> > res = python::object())
 {
-    res.reshapeIfEmpty(image.shape(), "distanceTransform2D(): Output array has wrong shape.");
+    res.reshapeIfEmpty(image.taggedShape(), 
+            "distanceTransform2D(): Output array has wrong shape.");
     
-    distanceTransform(srcImageRange(image), destImage(res), background, norm);
+    PyAllowThreads _pythread;
+    if(background)
+    {
+        distanceTransform(srcImageRange(image), destImage(res), 
+                          NumericTraits<PixelType>::zero(), norm);
+    }
+    else
+    {
+        distanceTransform(srcImageRange(image, detail::IsBackgroundAccessor<PixelType>()), 
+                          destImage(res), false, norm);
+    }
     return res;
 }
 
 template < class VoxelType >
 NumpyAnyArray 
 pythonDistanceTransform3D(NumpyArray<3, Singleband<VoxelType> > volume, 
-                          VoxelType background,
+                          bool background, 
                           NumpyArray<3, Singleband<VoxelType> > res=python::object())
 {
-    res.reshapeIfEmpty(volume.shape(), "distanceTransform3D(): Output array has wrong shape.");
+    res.reshapeIfEmpty(volume.taggedShape(), 
+            "distanceTransform3D(): Output array has wrong shape.");
     
+    PyAllowThreads _pythread;
     separableMultiDistance(srcMultiArrayRange(volume), destMultiArray(res), background);
     return res;
 }
@@ -616,16 +672,18 @@ void defineMorphology()
     def("distanceTransform2D",
         registerConverters(&pythonDistanceTransform2D<float, float>),
         (arg("image"), 
-         arg("background")=0, 
+         arg("background")=true, 
          arg("norm")=2,
          arg("out")=python::object()),
         "Compute the distance transform of a 2D scalar float image.\n"
-        "For all background pixels, calculate the distance to the nearest object or contour. "
-        "The label of the pixels to be considered background in the source image is passed "
-        "in the parameter 'background'. "
-        "Source pixels with other labels will be considered objects. "
-        "In the destination image, all pixels corresponding to background will hold "
-        "their distance value, all pixels corresponding to objects will be assigned 0.\n\n"
+        "All pixels with a value of 0.0 are considered to be background pixels,\n"
+        "while all pixels with a nonzero value are considered to be foreground pixels.\n"
+        "The parameter 'background' is a Boolean scalar that specifies whether to\n"
+        "compute the distance of all background pixels to the nearest foreground pixels\n"
+        "(if it is 'True', default) or vice versa (if it is 'False').\n"
+        "Hence in the destination image, for background==True all background pixels\n"
+        "will be assigned their distance value, while all foreground pixels will be assigned 0.\n"
+        "For background==False, it is exactly the other way around.\n\n"
         "The 'norm' parameter gives the distance norm to use\n"
         "(0: infinity norm, 1: L1 norm, 2: Euclidean norm).\n\n"
         "For details see distanceTransform_ in the vigra C++ documentation.\n");
@@ -633,21 +691,25 @@ void defineMorphology()
         def("distanceTransform2D",
         registerConverters(&pythonDistanceTransform2D<UInt8,float>),
         (arg("image"), 
-         arg("background")=0, 
+         arg("background")=true, 
          arg("norm")=2,
          arg("out")=python::object()),
         "Likewise for a 2D uint8 input array.\n");
 
     def("distanceTransform3D",
         registerConverters(&pythonDistanceTransform3D<float>),
-        (arg("array"), arg("background"), arg("out")=python::object()),
+        (arg("array"), 
+         arg("background") = true, 
+         arg("out")=python::object()),
         "Compute the Euclidean distance transform of a 3D scalar float volume.\n"
-        "For all background voxels, calculate the distance to the nearest object or contour."
-        "The label of the voxels to be considered background in the source volume is passed "
-        "in the parameter 'background'. "
-        "Source voxels with other labels will be considered objects. "
-        "In the destination volume, all voxels corresponding to background will be assigned "
-        "their distance value, all voxels corresponding to objects will be assigned 0.\n"
+        "All voxels with a value of 0.0 are considered to be background voxels,\n"
+        "while all voxels with a nonzero value are considered to be foreground voxels.\n"
+        "The parameter 'background' is a Boolean scalar that specifies whether to\n"
+        "compute the distance of all background voxels to the nearest foreground voxel\n"
+        "(if it is 'True', default) or vice versa (if it is 'False').\n"
+        "Hence in the destination volume, for background==True all background voxels\n"
+        "will be assigned their distance value, while all foreground voxels will be assigned 0.\n"
+        "For background==False, it is exactly the other way around.\n"
         "\n"
         "For more details see separableMultiDistance_ in the vigra C++ documentation.\n");
 }
