@@ -34,7 +34,7 @@
 /************************************************************************/
 
 #define PY_ARRAY_UNIQUE_SYMBOL vigranumpylearning_PyArray_API
-// #define NO_IMPORT_ARRAY
+#define NO_IMPORT_ARRAY
 
 #include <vigra/numpy_array.hxx>
 #include <vigra/numpy_array_converters.hxx>
@@ -42,6 +42,7 @@
 #ifdef HasHDF5
 # include <vigra/random_forest_hdf5_impex.hxx>
 #endif
+#include <vigra/timing.hxx>
 #include <set>
 #include <cmath>
 #include <memory>
@@ -52,9 +53,10 @@ namespace vigra
 {
 
 template<class FeatureType>
-OnlinePredictionSet<FeatureType>* pythonConstructOnlinePredictioSet(NumpyArray<2,FeatureType> features,int num_sets)
+OnlinePredictionSet<FeatureType>* 
+pythonConstructOnlinePredictioSet(NumpyArray<2,FeatureType> features, int num_sets)
 {
-    return new OnlinePredictionSet<FeatureType>(features,num_sets);
+    return new OnlinePredictionSet<FeatureType>(features, num_sets);
 }
 
 template<class LabelType, class FeatureType>
@@ -97,7 +99,7 @@ pythonConstructRandomForest(int treeCount,
 template<class LabelType>
 RandomForest<LabelType> * 
 pythonImportRandomForestFromHDF5(std::string filename, 
-					             std::string pathname = "")
+                                 std::string pathname = "")
 { 
     std::auto_ptr<RandomForest<LabelType> > rf(new RandomForest<LabelType>);
     
@@ -105,7 +107,7 @@ pythonImportRandomForestFromHDF5(std::string filename,
            "RandomForest(): Unable to load from HDF5 file.");
            
     return rf.release();
-}					
+}                   
 #endif // HasHDF5
 
 template<class LabelType, class FeatureType>
@@ -117,21 +119,17 @@ pythonLearnRandomForestWithFeatureSelection(RandomForest<LabelType> & rf,
     using namespace rf;
     visitors::VariableImportanceVisitor var_imp;
     visitors::OOB_Error                 oob_v;
-    double oob;
-	Py_BEGIN_ALLOW_THREADS
-	rf.learn(trainData, trainLabels, visitors::create_visitor(var_imp, oob_v));
-	Py_END_ALLOW_THREADS
-    oob = oob_v.oob_breiman;
+    
+    {
+        PyAllowThreads _pythread;
+        rf.learn(trainData, trainLabels, 
+                 visitors::create_visitor(var_imp, oob_v));
+    }
+    
+    double oob = oob_v.oob_breiman;
     // std::cout << "out of bag: " << oob << std::endl;
 
-    NumpyArray<2, double> varImp(MultiArrayShape<2>::type(var_imp.variable_importance_.shape(0),
-                                                           var_imp.variable_importance_.shape(1))); 
-
-    for (int x=0;x<varImp.shape(0);x++)
-        for (int y=0;y<varImp.shape(1);y++)
-            varImp(x,y)= var_imp.variable_importance_(x,y);
-
-    return python::make_tuple(oob, varImp);
+    return python::make_tuple(oob, var_imp.variable_importance_);
 }
 
 template<class LabelType, class FeatureType>
@@ -140,16 +138,17 @@ pythonLearnRandomForest(RandomForest<LabelType> & rf,
                         NumpyArray<2,FeatureType> trainData, 
                         NumpyArray<2,LabelType> trainLabels)
 {
-  using namespace rf;
-  double oob;
-  visitors::OOB_Error oob_v;
-  Py_BEGIN_ALLOW_THREADS
-  rf.learn(trainData, trainLabels, visitors::create_visitor(oob_v));
-  Py_END_ALLOW_THREADS
-  oob = oob_v.oob_breiman;
-  
-  //std::cout << "out of bag: " << oob << std::endl;
-  return oob;
+    using namespace rf;
+    visitors::OOB_Error oob_v;
+
+    {
+        PyAllowThreads _pythread;
+        rf.learn(trainData, trainLabels, visitors::create_visitor(oob_v));
+    }
+    double oob = oob_v.oob_breiman;
+
+    //std::cout << "out of bag: " << oob << std::endl;
+    return oob;
 }
 
 template<class LabelType,class FeatureType>
@@ -160,9 +159,8 @@ pythonRFOnlineLearn(RandomForest<LabelType> & rf,
                     int startIndex,
                     bool adjust_thresholds)
 {
-    Py_BEGIN_ALLOW_THREADS
+    PyAllowThreads _pythread;
     rf.onlineLearn(trainData, trainLabels, startIndex, adjust_thresholds);
-    Py_END_ALLOW_THREADS
 }
 
 template<class LabelType,class FeatureType>
@@ -172,9 +170,8 @@ pythonRFReLearnTree(RandomForest<LabelType> & rf,
                     NumpyArray<2,LabelType> trainLabels,
                     int treeId)
 {
-    Py_BEGIN_ALLOW_THREADS
-    rf.reLearnTree(trainData,trainLabels,treeId);
-    Py_END_ALLOW_THREADS
+    PyAllowThreads _pythread;
+    rf.reLearnTree(trainData, trainLabels, treeId);
 }
 
 template<class LabelType,class FeatureType>
@@ -183,10 +180,10 @@ pythonRFPredictLabels(RandomForest<LabelType> const & rf,
                       NumpyArray<2,FeatureType> testData,
                       NumpyArray<2,LabelType> res)
 {
-    //construct result
-    res.reshapeIfEmpty(MultiArrayShape<2>::type(testData.shape(0),1),
+    res.reshapeIfEmpty(MultiArrayShape<2>::type(testData.shape(0), 1),
                        "Output array has wrong dimensions.");
-    rf.predictLabels(testData,res);
+    PyAllowThreads _pythread;
+    rf.predictLabels(testData, res);
     return res;
 }
 
@@ -196,12 +193,12 @@ pythonRFPredictProbabilities(RandomForest<LabelType> & rf,
                              NumpyArray<2,FeatureType> testData, 
                              NumpyArray<2,float> res)
 {
-    //construct result
     res.reshapeIfEmpty(MultiArrayShape<2>::type(testData.shape(0), rf.ext_param_.class_count_),
                        "Output array has wrong dimensions.");
-	Py_BEGIN_ALLOW_THREADS
-    rf.predictProbabilities(testData,res);
-	Py_END_ALLOW_THREADS
+    
+    PyAllowThreads _pythread;
+    rf.predictProbabilities(testData, res);
+
     return res;
 }
 
@@ -211,15 +208,17 @@ pythonRFPredictProbabilitiesOnlinePredSet(RandomForest<LabelType> & rf,
                                           OnlinePredictionSet<FeatureType> & predSet,
                                           NumpyArray<2,float> res)
 {
-    //construct result
-    res.reshapeIfEmpty(MultiArrayShape<2>::type(predSet.features.shape(0),rf.ext_param_.class_count_),
+    res.reshapeIfEmpty(MultiArrayShape<2>::type(predSet.features.shape(0),
+                                                 rf.ext_param_.class_count_),
                        "Output array has wrong dimenstions.");
-    clock_t start=clock();
-	Py_BEGIN_ALLOW_THREADS
-    rf.predictProbabilities(predSet, res);
-    Py_END_ALLOW_THREADS
-    double duration=(clock()-start)/double(CLOCKS_PER_SEC);
-    std::cerr<<"Prediction Time: "<<duration<<std::endl;
+    USETICTOC;
+    TIC;
+    {
+        PyAllowThreads _pythread;
+        rf.predictProbabilities(predSet, res);
+    }
+    std::string t = TOCS;
+    std::cerr << "Prediction Time: " << t << std::endl;
     return res;
 }
 
@@ -326,30 +325,14 @@ void defineRandomForest()
              "Works only if forest has been created with prepare_online_learning=true. "
              "Needs the old training data and the new appened, starting at startIndex.\n\n")
 #ifdef HasHDF5
-        .def("writeHDF5", &rf_export_HDF5<UInt32>,
-             (arg("filename"), arg("pathInFile")="", arg("overwriteflag")=false),
+        .def("writeHDF5", (void (*)(const RandomForest<UInt32, ClassificationTag> &, std::string const &, std::string const &))&rf_export_HDF5,
+             (arg("filename"), arg("pathInFile")=""),
              "Store the random forest in the given HDF5 file 'filname' under the internal\n"
-             "path 'pathInFile'. If a dataset already exists, 'overwriteflag' determines\n"
-             "if the old data are overwritten.\n")
+             "path 'pathInFile'.\n")
 #endif // HasHDF5
         ;
 }
 
-
-void defineRandomForestOld();
-
-
 } // namespace vigra
 
 
-using namespace vigra;
-using namespace boost::python;
-
-BOOST_PYTHON_MODULE_INIT(learning)
-{
-    import_vigranumpy();
-    defineRandomForest();
-    defineRandomForestOld();
-}
-
-
