@@ -58,14 +58,19 @@ namespace vigra
 template < class PixelType >
 NumpyAnyArray 
 pythonLabelImage(NumpyArray<2, Singleband<PixelType> > image,
-                 int neighborhood = 4,
-                 NumpyArray<2, Singleband<npy_uint32> > res = python::object())
+                 int neighborhood = 4, 
+                 NumpyArray<2, Singleband<npy_uint32> > res = NumpyArray<2, Singleband<npy_uint32> >())
 {
     vigra_precondition(neighborhood == 4 || neighborhood == 8,
         "labelImage(): neighborhood must be 4 or 8.");
 
-    res.reshapeIfEmpty(image.shape(), "labelImage(): Output array has wrong shape.");
+    std::string description("connected components, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+            "labelImage(): Output array has wrong shape.");
 
+    PyAllowThreads _pythread;
     switch (neighborhood)
     {
         case 4:
@@ -90,13 +95,18 @@ NumpyAnyArray
 pythonLabelImageWithBackground(NumpyArray<2, Singleband<PixelType> > image,
                                int neighborhood = 4,
                                PixelType background_value = 0,
-                               NumpyArray<2, Singleband<npy_uint32> > res = python::object())
+                               NumpyArray<2, Singleband<npy_uint32> > res = NumpyArray<2, Singleband<npy_uint32> >())
 {
     vigra_precondition(neighborhood == 4 || neighborhood == 8,
         "labelImageWithBackground(): neighborhood must be 4 or 8.");
 
-    res.reshapeIfEmpty(image.shape(), "labelImageWithBackground(): Output array has wrong shape.");
+    std::string description("connected components with background, neighborhood=");
+    description += asString(neighborhood)+ ", bglabel=" + asString(background_value);
+    
+    res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+        "labelImageWithBackground(): Output array has wrong shape.");
 
+    PyAllowThreads _pythread;
     switch (neighborhood)
     {
         case 4:
@@ -121,13 +131,18 @@ template < class VoxelType >
 NumpyAnyArray 
 pythonLabelVolume(NumpyArray<3, Singleband<VoxelType> > volume, 
                   int neighborhood=6,
-                  NumpyArray<3, Singleband<npy_uint32> > res=python::object())
+                  NumpyArray<3, Singleband<npy_uint32> > res = NumpyArray<3, Singleband<npy_uint32> >())
 {
     vigra_precondition(neighborhood == 6 || neighborhood == 26,
         "labelVolume(): neighborhood must be 6 or 26.");
     
-    res.reshapeIfEmpty(volume.shape(), "labelVolume(): Output array has wrong shape.");
+    std::string description("connected components, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(volume.taggedShape().setChannelDescription(description), 
+            "labelVolume(): Output array has wrong shape.");
 
+    PyAllowThreads _pythread;
     switch (neighborhood)
     {
         case 6:
@@ -153,12 +168,18 @@ NumpyAnyArray
 pythonLabelVolumeWithBackground(NumpyArray<3, Singleband<VoxelType> > volume, 
                                 int neighborhood=6,
                                 VoxelType background_value = 0,
-                                NumpyArray<3, Singleband<npy_uint32> > res=python::object())
+                                NumpyArray<3, Singleband<npy_uint32> > res = NumpyArray<3, Singleband<npy_uint32> >())
 {
     vigra_precondition(neighborhood == 6 || neighborhood == 26,
         "labelVolumeWithBackground(): neighborhood must be 6 or 26.");
     
-    res.reshapeIfEmpty(volume.shape(), "labelVolumeWithBackground(): Output array has wrong shape.");
+    std::string description("connected components with background, neighborhood=");
+    description += asString(neighborhood)+ ", bglabel=" + asString(background_value);
+    
+    res.reshapeIfEmpty(volume.taggedShape().setChannelDescription(description), 
+        "labelVolumeWithBackground(): Output array has wrong shape.");
+
+    PyAllowThreads _pythread;
     switch (neighborhood)
     {
         case 6:
@@ -188,12 +209,18 @@ NumpyAnyArray
 pythonLocalMinima2D(NumpyArray<2, Singleband<PixelType> > image,
                     PixelType marker = NumericTraits<PixelType>::one(),
                     int neighborhood = 8,
-                    NumpyArray<2, Singleband<PixelType> > res = python::object())
+                    NumpyArray<2, Singleband<PixelType> > res = NumpyArray<2, Singleband<PixelType> >())
 {
     vigra_precondition(neighborhood == 4 || neighborhood == 8,
         "localMinima(): neighborhood must be 4 or 8.");
 
-    res.reshapeIfEmpty(image.shape(), "localMinima(): Output array has wrong shape.");
+    std::string description("local minima, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+            "localMinima(): Output array has wrong shape.");
+
+    PyAllowThreads _pythread;
     switch (neighborhood)
     {
         case 4:
@@ -213,17 +240,58 @@ pythonLocalMinima2D(NumpyArray<2, Singleband<PixelType> > image,
     return res;
 }
 
+template<class PixelType>
+NumpyAnyArray 
+pythonLocalMinima3D(NumpyArray<3, Singleband<PixelType> > volume,
+                    PixelType marker = NumericTraits<PixelType>::one(), 
+                    int neighborhood = 6, 
+                    NumpyArray<3, Singleband<PixelType> > res = NumpyArray<3, Singleband<PixelType> >())
+{
+    vigra_precondition(neighborhood == 6 || neighborhood == 26,
+            "localMinima(): neighborhood must be 6 or 26.");
+
+    std::string description("local minima, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(volume.taggedShape().setChannelDescription(description), 
+            "localMinima(): Output array has wrong shape.");
+            
+    switch (neighborhood)
+    {
+        case 6:
+        {
+            localMinima3D(srcMultiArrayRange(volume), destMultiArray(res), marker,
+                    NeighborCode3DSix());
+            break;
+        }
+        case 26:
+        {
+            localMinima3D(srcMultiArrayRange(volume), destMultiArray(res), marker,
+                    NeighborCode3DTwentySix());
+            break;
+        }
+    }
+
+    return res;
+}
+
 template < class PixelType >
 NumpyAnyArray 
 pythonExtendedLocalMinima2D(NumpyArray<2, Singleband<PixelType> > image,
                             PixelType marker = NumericTraits<PixelType>::one(),
                             int neighborhood = 8,
-                            NumpyArray<2, Singleband<PixelType> > res = python::object())
+                            NumpyArray<2, Singleband<PixelType> > res = NumpyArray<2, Singleband<PixelType> >())
 {
     vigra_precondition(neighborhood == 4 || neighborhood == 8,
         "extendedLocalMinima(): neighborhood must be 4 or 8.");
 
-    res.reshapeIfEmpty(image.shape(), "extendedLocalMinima(): Output array has wrong shape.");
+    std::string description("extended local minima, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+        "extendedLocalMinima(): Output array has wrong shape.");
+
+    PyAllowThreads _pythread;
     switch (neighborhood)
     {
         case 4:
@@ -242,17 +310,57 @@ pythonExtendedLocalMinima2D(NumpyArray<2, Singleband<PixelType> > image,
     return res;
 }
 
+template<class PixelType>
+NumpyAnyArray 
+pythonExtendedLocalMinima3D(NumpyArray<3, Singleband<PixelType> > volume, 
+                            PixelType marker = NumericTraits<PixelType>::one(), 
+                            int neighborhood = 6,
+                            NumpyArray<3, Singleband<PixelType> > res = NumpyArray<3, Singleband<PixelType> >())
+{
+    vigra_precondition(neighborhood == 6 || neighborhood == 26,
+            "extendedLocalMinima(): neighborhood must be 6 or 26.");
+
+    std::string description("extended local minima, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(volume.taggedShape().setChannelDescription(description), 
+            "extendedLocalMinima(): Output array has wrong shape.");
+    switch (neighborhood)
+    {
+        case 6:
+        {
+            extendedLocalMinima3D(srcMultiArrayRange(volume), destMultiArray(res),
+                marker, NeighborCode3DSix());
+            break;
+        }
+        case 26:
+        {
+            extendedLocalMinima3D(srcMultiArrayRange(volume), destMultiArray(res),
+                marker, NeighborCode3DTwentySix());
+            break;
+        }
+    }
+
+    return res;
+}
+
 template < class PixelType >
 NumpyAnyArray 
 pythonLocalMaxima2D(NumpyArray<2, Singleband<PixelType> > image,
                     PixelType marker = NumericTraits<PixelType>::one(),
                     int neighborhood = 8,
-                    NumpyArray<2, Singleband<PixelType> > res = python::object())
+                    NumpyArray<2, Singleband<PixelType> > res = NumpyArray<2, Singleband<PixelType> >())
 {
     vigra_precondition(neighborhood == 4 || neighborhood == 8,
         "localMaxima(): neighborhood must be 4 or 8.");
 
-    res.reshapeIfEmpty(image.shape(), "localMaxima(): Output array has wrong shape.");
+    std::string description("local maxima, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+            "localMaxima(): Output array has wrong shape.");
+
+    PyAllowThreads _pythread;
     switch (neighborhood)
     {
         case 4:
@@ -272,17 +380,57 @@ pythonLocalMaxima2D(NumpyArray<2, Singleband<PixelType> > image,
     return res;
 }
 
+template<class PixelType>
+NumpyAnyArray 
+pythonLocalMaxima3D(NumpyArray<3, Singleband<PixelType> > volume,
+                    PixelType marker = NumericTraits<PixelType>::one(), 
+                    int neighborhood = 6, 
+                    NumpyArray<3, Singleband<PixelType> > res = NumpyArray<3, Singleband<PixelType> >())
+{
+    vigra_precondition(neighborhood == 6 || neighborhood == 26,
+            "localMaxima(): neighborhood must be 6 or 26.");
+
+    std::string description("local maxima, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(volume.taggedShape().setChannelDescription(description), 
+            "localMaxima(): Output array has wrong shape.");
+    switch (neighborhood)
+    {
+        case 6:
+        {
+            localMaxima3D(srcMultiArrayRange(volume), destMultiArray(res), marker,
+                NeighborCode3DSix());
+            break;
+        }
+        case 26:
+        {
+            localMaxima3D(srcMultiArrayRange(volume), destMultiArray(res), marker,
+                NeighborCode3DTwentySix());
+            break;
+        }
+    }
+
+    return res;
+}
+
 template < class PixelType >
 NumpyAnyArray 
 pythonExtendedLocalMaxima2D(NumpyArray<2, Singleband<PixelType> > image,
                             PixelType marker = NumericTraits<PixelType>::one(),
                             int neighborhood = 8,
-                            NumpyArray<2, Singleband<PixelType> > res = python::object())
+                            NumpyArray<2, Singleband<PixelType> > res = NumpyArray<2, Singleband<PixelType> >())
 {
     vigra_precondition(neighborhood == 4 || neighborhood == 8,
         "extendedLocalMaxima(): neighborhood must be 4 or 8.");
 
-    res.reshapeIfEmpty(image.shape(), "extendedLocalMaxima(): Output array has wrong shape.");
+    std::string description("extended local maxima, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+            "extendedLocalMaxima(): Output array has wrong shape.");
+
+    PyAllowThreads _pythread;
     switch (neighborhood)
     {
         case 4:
@@ -301,17 +449,52 @@ pythonExtendedLocalMaxima2D(NumpyArray<2, Singleband<PixelType> > image,
     return res;
 }
 
+template<class PixelType>
+NumpyAnyArray 
+pythonExtendedLocalMaxima3D(NumpyArray<3, Singleband<PixelType> > volume, 
+                            PixelType marker = NumericTraits<PixelType>::one(), 
+                            int neighborhood = 6,
+                            NumpyArray<3, Singleband<PixelType> > res = NumpyArray<3, Singleband<PixelType> >())
+{
+    vigra_precondition(neighborhood == 6 || neighborhood == 26,
+            "extendedLocalMaxima(): neighborhood must be 6 or 26.");
+
+    std::string description("extended local maxima, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(volume.taggedShape().setChannelDescription(description), 
+            "extendedLocalMaxima(): Output array has wrong shape.");
+    switch (neighborhood)
+    {
+        case 6:
+        {
+            extendedLocalMaxima3D(srcMultiArrayRange(volume), destMultiArray(res),
+                                  marker, NeighborCode3DSix());
+            break;
+        }
+        case 26:
+        {
+            extendedLocalMaxima3D(srcMultiArrayRange(volume), destMultiArray(res),
+                                  marker, NeighborCode3DTwentySix());
+            break;
+        }
+    }
+
+    return res;
+}
+
 /*************************************************************************/
 
+#if 0
 template < class PixelType >
 python::tuple 
-pythonWatersheds2D(NumpyArray<2, Singleband<PixelType> > image,
+pythonWatersheds2DOld(NumpyArray<2, Singleband<PixelType> > image,
                    int neighborhood = 4,
                    NumpyArray<2, Singleband<npy_uint32> > seeds = python::object(),
                    std::string method = "RegionGrowing", 
                    SRGType srgType = CompleteGrow, 
                    PixelType max_cost = 0.0, 
-                   NumpyArray<2, Singleband<npy_uint32> > res = python::object())
+                   NumpyArray<2, Singleband<npy_uint32> > res = NumpyArray<2, Singleband<npy_uint32> >())
 {
     vigra_precondition(neighborhood == 4 || neighborhood == 8,
            "watersheds2D(): neighborhood must be 4 or 8.");
@@ -327,7 +510,8 @@ pythonWatersheds2D(NumpyArray<2, Singleband<PixelType> > image,
     
     if(method == "regiongrowing")
     {
-        seeds.reshapeIfEmpty(image.shape(), "watersheds(): Seed array has wrong shape.");
+        seeds.reshapeIfEmpty(image.shape(), 
+                "watersheds(): Seed array has wrong shape.");
         
         if(!haveSeeds)
         {
@@ -367,12 +551,98 @@ pythonWatersheds2D(NumpyArray<2, Singleband<PixelType> > image,
         
         if(neighborhood == 4)
         {
-            maxRegionLabel = watersheds(srcImageRange(image), destImage(res),
+            maxRegionLabel = watershedsUnionFind(srcImageRange(image), destImage(res),
                                         FourNeighborCode());
         }
         else
         {
-            maxRegionLabel = watersheds(srcImageRange(image), destImage(res),
+            maxRegionLabel = watershedsUnionFind(srcImageRange(image), destImage(res),
+                                        EightNeighborCode());
+        }
+    }
+    else
+    {
+        vigra_precondition(false, "watersheds(): Unknown watershed method requested.");
+    }
+
+    return python::make_tuple(res, maxRegionLabel);
+}
+#endif
+
+template < class PixelType >
+python::tuple 
+pythonWatersheds2D(NumpyArray<2, Singleband<PixelType> > image,
+                   int neighborhood = 4,
+                   NumpyArray<2, Singleband<npy_uint32> > seeds = python::object(),
+                   std::string method = "RegionGrowing", 
+                   SRGType srgType = CompleteGrow, 
+                   PixelType max_cost = 0.0, 
+                   NumpyArray<2, Singleband<npy_uint32> > res = NumpyArray<2, Singleband<npy_uint32> >())
+{
+    vigra_precondition(neighborhood == 4 || neighborhood == 8,
+           "watersheds2D(): neighborhood must be 4 or 8.");
+
+    for(unsigned int k=0; k<method.size(); ++k)
+        method[k] = (std::string::value_type)tolower(method[k]);
+    if(method == "")
+        method = "regiongrowing";
+        
+    std::string description("watershed labeling, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+            "watersheds(): Output array has wrong shape.");
+    
+    WatershedOptions options;
+    options.srgType(srgType);
+    
+    if(max_cost > 0.0)
+    {
+        vigra_precondition(method != "unionfind",
+           "watersheds(): UnionFind does not support a cost threshold.");
+        options.stopAtThreshold(max_cost);
+    }
+    
+    if(seeds.hasData())
+    {
+        vigra_precondition(method != "unionfind",
+           "watersheds(): UnionFind does not support seed images.");
+        res = seeds;
+    }
+    else
+    {
+        options.seedOptions(SeedOptions().minima());
+    }
+    
+    npy_uint32 maxRegionLabel = 0;
+    if(method == "regiongrowing")
+    {
+        PyAllowThreads _pythread;
+        if(neighborhood == 4)
+        {
+            maxRegionLabel = watershedsRegionGrowing(srcImageRange(image), destImage(res), 
+                                    FourNeighborCode(), options);
+        }
+        else
+        {
+            maxRegionLabel = watershedsRegionGrowing(srcImageRange(image), destImage(res), 
+                                    EightNeighborCode(), options);
+        }
+    }
+    else if(method == "unionfind")
+    {
+        vigra_precondition(srgType == CompleteGrow,
+           "watersheds(): UnionFind only supports 'CompleteGrow' mode.");
+           
+        PyAllowThreads _pythread;
+        if(neighborhood == 4)
+        {
+            maxRegionLabel = watershedsUnionFind(srcImageRange(image), destImage(res),
+                                        FourNeighborCode());
+        }
+        else
+        {
+            maxRegionLabel = watershedsUnionFind(srcImageRange(image), destImage(res),
                                         EightNeighborCode());
         }
     }
@@ -394,7 +664,7 @@ pythonWatersheds3D(NumpyArray<3, Singleband<PixelType> > image,
                    std::string method = "RegionGrowing", 
                    SRGType srgType = CompleteGrow, 
                    PixelType max_cost = 0.0, 
-                   NumpyArray<3, Singleband<npy_uint32> > res = python::object())
+                   NumpyArray<3, Singleband<npy_uint32> > res = NumpyArray<3,Singleband<npy_uint32> >())
 {
     vigra_precondition(neighborhood == 6 || neighborhood == 26,
            "watersheds3D(): neighborhood must be 6 or 26.");
@@ -410,10 +680,14 @@ pythonWatersheds3D(NumpyArray<3, Singleband<PixelType> > image,
     
     if(method == "regiongrowing")
     {
-        seeds.reshapeIfEmpty(image.shape(), "watersheds(): Seed array has wrong shape.");
+        std::string description("watershed seeds");
+        
+        seeds.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+                "watersheds(): Seed array has wrong shape.");
         
         if(!haveSeeds)
         {
+            PyAllowThreads _pythread;
             maxRegionLabel = 0;
             
             // determine seeds
@@ -463,22 +737,30 @@ pythonWatersheds3D(NumpyArray<3, Singleband<PixelType> > image,
         }
         else
         {
+            PyAllowThreads _pythread;
             FindMinMax< npy_uint32 > minmax;
             inspectMultiArray(srcMultiArrayRange(seeds), minmax);
             maxRegionLabel = minmax.max;
         }
            
-        res.reshapeIfEmpty(image.shape(), "watersheds(): Output array has wrong shape.");
+        description = "watershed labeling, neighborhood=";
+        description += asString(neighborhood);
+        
+        res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+                "watersheds(): Output array has wrong shape.");
 
+        PyAllowThreads _pythread;
         ArrayOfRegionStatistics< SeedRgDirectValueFunctor< PixelType > > stats(maxRegionLabel);
         if(neighborhood == 6)
         {
-            seededRegionGrowing3D(srcMultiArrayRange(image), srcMultiArray(seeds), destMultiArray(res), 
+            seededRegionGrowing3D(srcMultiArrayRange(image), srcMultiArray(seeds), 
+                                  destMultiArray(res), 
                                   stats, srgType, NeighborCode3DSix(), max_cost);
         }
         else
         {
-            seededRegionGrowing3D(srcMultiArrayRange(image), srcMultiArray(seeds), destMultiArray(res), 
+            seededRegionGrowing3D(srcMultiArrayRange(image), srcMultiArray(seeds), 
+                                  destMultiArray(res), 
                                   stats, srgType, NeighborCode3DTwentySix(), max_cost);
         }
     }
@@ -489,8 +771,13 @@ pythonWatersheds3D(NumpyArray<3, Singleband<PixelType> > image,
         vigra_precondition(srgType == CompleteGrow,
            "watersheds(): UnionFind only supports 'CompleteGrow' mode.");
            
-        res.reshapeIfEmpty(image.shape(), "watersheds(): Output array has wrong shape.");
+        std::string description("watershed labeling, neighborhood=");
+        description += asString(neighborhood);
         
+        res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+                "watersheds(): Output array has wrong shape.");
+        
+        PyAllowThreads _pythread;
         if(neighborhood == 6)
         {
             maxRegionLabel = watersheds3DSix(srcMultiArrayRange(image), destMultiArray(res));
@@ -569,6 +856,15 @@ void defineSegmentation()
         "4 or 8 (default).\n\n"
         "For details see localMinima_ in the vigra C++ documentation.\n");
 
+    def("localMinima3D",
+            registerConverters(&pythonLocalMinima3D<float> ),
+            (arg("volume"), arg("marker") = 1.0, arg("neighborhood") = 6, arg(
+                    "out") = python::object()),
+            "Find local minima in a volume and mark them with the given 'marker'. Parameter "
+                "'neighborhood' specifies the pixel neighborhood to be used and can be "
+                "6 or 26 (default).\n\n"
+                "For details see localMinima3D_ in the vigra C++ documentation.\n");
+
     def("extendedLocalMinima",
         registerConverters(&pythonExtendedLocalMinima2D<float>),
         (arg("image"), 
@@ -581,6 +877,14 @@ void defineSegmentation()
         "For details see extendedLocalMinima_ in the vigra C++ documentation.\n"
         );
 
+    def("extendedLocalMinima3D", 
+         registerConverters(&pythonExtendedLocalMinima3D<float> ), 
+           (arg("volume"), arg("marker") = 1.0, arg("neighborhood") = 6, arg("out") = python::object()),
+            "Find local minima and minimal plateaus in a volume and mark them with "
+            "the given 'marker'. Parameter 'neighborhood' specifies the pixel "
+            "neighborhood to be used and can be 6(default) or 26 .\n\n"
+            "For details see extendedLocalMinima3D_ in the vigra C++ documentation.\n");
+
     def("localMaxima",
         registerConverters(&pythonLocalMaxima2D<float>),
         (arg("image"), 
@@ -592,6 +896,13 @@ void defineSegmentation()
         "4 or 8 (default).\n\n"
         "For details see localMaxima_ in the vigra C++ documentation.\n");
 
+    def("localMaxima3D", registerConverters(&pythonLocalMaxima3D<float> ), 
+         (arg("volume"), arg("marker") = 1.0, arg("neighborhood") = 6, arg("out") = python::object()),
+            "Find local maxima and maximal plateaus in a volume and mark them with "
+            "the given 'marker'. Parameter 'neighborhood' specifies the pixel "
+            "neighborhood to be used and can be 6(default) or 26 .\n\n"
+            "For details see localMaxima3D_ in the vigra C++ documentation.\n");
+
     def("extendedLocalMaxima",
         registerConverters(&pythonExtendedLocalMaxima2D<float>),
         (arg("image"), 
@@ -601,7 +912,15 @@ void defineSegmentation()
         "Find local maxima and maximal plateaus in an image and mark them with "
         "the given 'marker'. Parameter 'neighborhood' specifies the pixel "
         "neighborhood to be used and can be 4 or 8 (default).\n\n"
-        "For details see localMinima_ in the vigra C++ documentation.\n");
+        "For details see extendedLocalMaxima_ in the vigra C++ documentation.\n");
+
+    def("extendedLocalMaxima3D", 
+        registerConverters(&pythonExtendedLocalMaxima3D<float> ), 
+        (arg("volume"), arg("marker") = 1.0, arg("neighborhood") = 6, arg("out") = python::object()),
+        "Find local maxima and maximal plateaus in a volume and mark them with "
+        "the given 'marker'. Parameter 'neighborhood' specifies the pixel "
+        "neighborhood to be used and can be 6 (default) or 26 .\n\n"
+        "For details see extendedLocalMaxima3D_ in the vigra C++ documentation.\n");
 
     /*************************************************************************/
 
@@ -644,7 +963,7 @@ void defineSegmentation()
         "      'RegionGrowing':\n"
         "        (default) use seededRegionGrowing_ or seededRegionGrowing3D_ respectively\n"
         "      'UnionFind:\n"
-        "        use watersheds_ or watersheds3D_ respectively\n\n"
+        "        use watershedsUnionFind_ or watersheds3D_ respectively\n\n"
         " terminate:\n"
         "    when to stop growing. Possible values:\n\n"
         "      CompleteGrow:\n"
