@@ -40,12 +40,23 @@
 #include <boost/python.hpp>
 #include <vigra/numpy_array.hxx>
 #include <vigra/numpy_array_converters.hxx>
+#include <vigra/functorexpression.hxx>
+#include <vigra/mathutil.hxx>
+#include <vigra/utilities.hxx>
+#include <vector>
 
-NUMPY_ARRAY_INITIALIZE_REGISTRY
+namespace python = boost::python;
 
 namespace vigra {
 
+UInt32 pychecksum(python::str const & s)
+{
+    unsigned int size = len(s);
+    return checksum(PyString_AsString(s.ptr()), size);
+}
+
 void registerNumpyArrayConverters();
+void defineAxisTags();
 
 } // namespace vigra
 
@@ -56,4 +67,7 @@ BOOST_PYTHON_MODULE_INIT(vigranumpycore)
 {
     import_array();
     registerNumpyArrayConverters();
+    defineAxisTags();
+    
+    def("checksum", &pychecksum, args("data"));
 }
